package digital.laboratory.platform.sys.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.sys.entity.Business;
import digital.laboratory.platform.sys.service.BusinessService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;


/**
 * 业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用
 *
 * @author Zhang Xiaolong created at  2022-07-15
 * @describe 业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用 前端控制器
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/business" )
@Api(value = "business", tags = "业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用管理")
public class CommBusinessController {

    private final BusinessService businessService;

	/**
	 * 通过id查询业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用
	 * @param id id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{id}" )
	@PreAuthorize("@pms.hasPermission('common_business_get')" )
	public R getById(@PathVariable("id" ) String id) {
		return R.ok(businessService.getById(id));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param business 业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('common_business_get')" )
    public R getBusinessPage(Page page, Business business) {
        return R.ok(businessService.page(page, Wrappers.query(business)));
    }


    /**
     * 新增业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用
     * @param business 业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用
     * @return R
     */
    @ApiOperation(value = "新增业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用", notes = "新增业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用")
    @SysLog("新增业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('common_business_add')" )
    public R postAddObject(@RequestBody Business business) {
        return R.ok(businessService.save(business));
    }

    /**
     * 修改业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用
     * @param business 业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用
     * @return R
     */
    @ApiOperation(value = "修改业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用", notes = "修改业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用")
    @SysLog("修改业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('common_business_edit')" )
    public R putUpdateById(@RequestBody Business business) {
        return R.ok(businessService.updateById(business));
    }

    /**
     * 通过id删除业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用", notes = "通过id删除业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用")
    @SysLog("通过id删除业务分类,目前只有:司法鉴定、污水检测任务、先进性检测任务。数据中的id不能变,代码中会直接使用" )
    @DeleteMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('common_business_del')" )
    public R deleteById(@PathVariable String id) {
        return R.ok(businessService.removeById(id));
    }

}
