package digital.laboratory.platform.sys.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.sys.entity.BusinessDepartment;
import digital.laboratory.platform.sys.service.BusinessDepartmentService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;


/**
 * 鉴定中心的业务部门设置
 *
 * @author Zhang Xiaolong created at  2022-07-15
 * @describe 鉴定中心的业务部门设置 前端控制器
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/business_department" )
@Api(value = "business_department", tags = "鉴定中心的业务部门设置管理")
public class CommBusinessDepartmentController {

    private final BusinessDepartmentService businessDepartmentService;

	/**
	 * 通过id查询鉴定中心的业务部门设置
	 * @param id id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{id}" )
	@PreAuthorize("@pms.hasPermission('common_business_department_get')" )
	public R getById(@PathVariable("id" ) String id) {
		return R.ok(businessDepartmentService.getById(id));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param businessDepartment 鉴定中心的业务部门设置
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('common_business_department_get')" )
    public R getBusinessDepartmentPage(Page page, BusinessDepartment businessDepartment) {
        return R.ok(businessDepartmentService.page(page, Wrappers.query(businessDepartment)));
    }


    /**
     * 新增鉴定中心的业务部门设置
     * @param businessDepartment 鉴定中心的业务部门设置
     * @return R
     */
    @ApiOperation(value = "新增鉴定中心的业务部门设置", notes = "新增鉴定中心的业务部门设置")
    @SysLog("新增鉴定中心的业务部门设置" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('common_business_department_add')" )
    public R postAddObject(@RequestBody BusinessDepartment businessDepartment) {
        return R.ok(businessDepartmentService.save(businessDepartment));
    }

    /**
     * 修改鉴定中心的业务部门设置
     * @param businessDepartment 鉴定中心的业务部门设置
     * @return R
     */
    @ApiOperation(value = "修改鉴定中心的业务部门设置", notes = "修改鉴定中心的业务部门设置")
    @SysLog("修改鉴定中心的业务部门设置" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('common_business_department_edit')" )
    public R putUpdateById(@RequestBody BusinessDepartment businessDepartment) {
        return R.ok(businessDepartmentService.updateById(businessDepartment));
    }

    /**
     * 通过id删除鉴定中心的业务部门设置
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除鉴定中心的业务部门设置", notes = "通过id删除鉴定中心的业务部门设置")
    @SysLog("通过id删除鉴定中心的业务部门设置" )
    @DeleteMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('common_business_department_del')" )
    public R deleteById(@PathVariable String id) {
        return R.ok(businessDepartmentService.removeById(id));
    }

}
