package digital.laboratory.platform.sys.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.sys.entity.BusinessRDepartment;
import digital.laboratory.platform.sys.service.BusinessRDepartmentService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;


/**
 * 业务与专业检验室的关系
 *
 * @author Zhang Xiaolong created at  2022-07-15
 * @describe 业务与专业检验室的关系 前端控制器
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/business_r_department" )
@Api(value = "business_r_department", tags = "业务与专业检验室的关系管理")
public class CommBusinessRDepartmentController {

    private final BusinessRDepartmentService businessRDepartmentService;

	/**
	 * 通过id查询业务与专业检验室的关系
	 * @param departmentId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{departmentId}" )
	@PreAuthorize("@pms.hasPermission('common_business_r_department_get')" )
	public R getById(@PathVariable("departmentId" ) String departmentId) {
		return R.ok(businessRDepartmentService.getById(departmentId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param businessRDepartment 业务与专业检验室的关系
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('common_business_r_department_get')" )
    public R getBusinessRDepartmentPage(Page page, BusinessRDepartment businessRDepartment) {
        return R.ok(businessRDepartmentService.page(page, Wrappers.query(businessRDepartment)));
    }


    /**
     * 新增业务与专业检验室的关系
     * @param businessRDepartment 业务与专业检验室的关系
     * @return R
     */
    @ApiOperation(value = "新增业务与专业检验室的关系", notes = "新增业务与专业检验室的关系")
    @SysLog("新增业务与专业检验室的关系" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('common_business_r_department_add')" )
    public R postAddObject(@RequestBody BusinessRDepartment businessRDepartment) {
        return R.ok(businessRDepartmentService.save(businessRDepartment));
    }

    /**
     * 修改业务与专业检验室的关系
     * @param businessRDepartment 业务与专业检验室的关系
     * @return R
     */
    @ApiOperation(value = "修改业务与专业检验室的关系", notes = "修改业务与专业检验室的关系")
    @SysLog("修改业务与专业检验室的关系" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('common_business_r_department_edit')" )
    public R putUpdateById(@RequestBody BusinessRDepartment businessRDepartment) {
        return R.ok(businessRDepartmentService.updateById(businessRDepartment));
    }

    /**
     * 通过id删除业务与专业检验室的关系
     * @param departmentId id
     * @return R
     */
    @ApiOperation(value = "通过id删除业务与专业检验室的关系", notes = "通过id删除业务与专业检验室的关系")
    @SysLog("通过id删除业务与专业检验室的关系" )
    @DeleteMapping("/{departmentId}" )
    @PreAuthorize("@pms.hasPermission('common_business_r_department_del')" )
    public R deleteById(@PathVariable String departmentId) {
        return R.ok(businessRDepartmentService.removeById(departmentId));
    }

}
