package digital.laboratory.platform.sys.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.sys.entity.DrugRType;
import digital.laboratory.platform.sys.service.DrugRTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;


/**
 * 毒品-类型关系映射表
 *
 * @author Zhang Xiaolong created at  2022-06-23
 * @describe 毒品-类型关系映射表 前端控制器
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/comm/drug_r_type" )
@Api(value = "drug_r_type", tags = "毒品-类型关系映射表管理")
public class CommDrugRTypeController {

    private final DrugRTypeService drugRTypeService;

	/**
	 * 通过id查询毒品-类型关系映射表
	 * @param drugId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{drugId}" )
//	@PreAuthorize("@pms.hasPermission('common_drug_r_type_get')" )
	public R getById(@PathVariable("drugId" ) String drugId) {
		return R.ok(drugRTypeService.getById(drugId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param drugRType 毒品-类型关系映射表
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
//    @PreAuthorize("@pms.hasPermission('common_drug_r_type_get')" )
    public R getDrugRTypePage(Page page, DrugRType drugRType) {
        return R.ok(drugRTypeService.page(page, Wrappers.query(drugRType)));
    }


    /**
     * 新增毒品-类型关系映射表
     * @param drugRType 毒品-类型关系映射表
     * @return R
     */
    @ApiOperation(value = "新增毒品-类型关系映射表", notes = "新增毒品-类型关系映射表")
    @SysLog("新增毒品-类型关系映射表" )
    @PostMapping
//    @PreAuthorize("@pms.hasPermission('common_drug_r_type_add')" )
    public R postAddObject(@RequestBody DrugRType drugRType) {
        return R.ok(drugRTypeService.save(drugRType));
    }

    /**
     * 修改毒品-类型关系映射表
     * @param drugRType 毒品-类型关系映射表
     * @return R
     */
    @ApiOperation(value = "修改毒品-类型关系映射表", notes = "修改毒品-类型关系映射表")
    @SysLog("修改毒品-类型关系映射表" )
    @PutMapping
//    @PreAuthorize("@pms.hasPermission('common_drug_r_type_edit')" )
    public R putUpdateById(@RequestBody DrugRType drugRType) {
        return R.ok(drugRTypeService.updateById(drugRType));
    }

    /**
     * 通过id删除毒品-类型关系映射表
     * @param drugId id
     * @return R
     */
    @ApiOperation(value = "通过id删除毒品-类型关系映射表", notes = "通过id删除毒品-类型关系映射表")
    @SysLog("通过id删除毒品-类型关系映射表" )
    @DeleteMapping("/{drugId}" )
//    @PreAuthorize("@pms.hasPermission('common_drug_r_type_del')" )
    public R deleteById(@PathVariable String drugId) {
        return R.ok(drugRTypeService.removeById(drugId));
    }

}
