package digital.laboratory.platform.sys.controller;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.sys.dto.UserDTO;
import digital.laboratory.platform.sys.entity.SysUser;
import digital.laboratory.platform.sys.service.StaffService;
import digital.laboratory.platform.sys.service.SysUserService;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.sys.vo.Staff;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * 涉及用户的公共服务
 */

@RestController
@RequiredArgsConstructor
@RequestMapping("/comm/user")
@Api(tags = "04-涉及用户的公共服务")
public class CommUserController {

	private final SysUserService userService;
	private final StaffService staffService;
//	private final SysRolePermissionService rolePermissionService;


	/**
	 * 查询对指定的机构拥有某些权限的用户列表。
	 *
	 * 还可以通过 includeSubOrg, 对用户进行筛选。
	 * includeSubOrg 为 true 时, 如果指定机构是某用户所属机构的下属机构时该用户有效;
	 * includeSubOrg 为 false 时, 只有当指定机构是某用户所属机构时该用户有效;
	 * @param orgId 机构 id
	 * param includeSubOrg 是否包含子机构
	 * @param permissions 权限集合
	 * @return 用户列表
	 */
	@GetMapping("/bypermissions")
	public R<List<SysUser>> GetUsersByPermission(@RequestParam("orgId") String orgId, @RequestParam("permissions") List<String> permissions) {
		//rolePermissionService
		return R.ok(userService.listUsersByPermission(orgId, false, permissions));
	}

	/**
	 * 分页查询用户
	 * @param page 参数集
	 * @param userDTO 查询参数列表
	 * @return 用户集合
	 */
	@GetMapping("/page")
	public R getUserPage(Page page, UserDTO userDTO) {
		return R.ok(userService.getUserWithRolePage(page, userDTO));
	}

	/**
	 * 注册新用户接口
	 * @param userDto
	 * @return
	 */
	@ApiOperation(value = "注册新用户",notes = "用于开放注册时，调用的接口")
	@PostMapping("/registerUser")
	public R registerUser(@Valid @RequestBody UserDTO userDto){
		return R.ok(userService.registerUser(userDto));
	}

	//前端登录修改用户信息
	@ApiOperation(value = "前端登录修改自己的基本信息",notes = "前端登录修改自己的基本信息")
	@PostMapping("/updateFrontUserInfo")
	public R updateFrontUserInfo(@Valid @RequestBody UserDTO userDto){
		Assert.notBlank(userDto.getUserId(),"用户ID不能为空");
		return R.ok(userService.updateFrontUserInfo(userDto),"数据操作成功");
	}
//	@ApiOperation(value = "快速创建特殊工作人员",notes = "快速创建特殊工作人员")
	@PostMapping("/addSpecialUserInfo")
	public R addSpecialUserInfo(@Valid @RequestBody List<UserDTO> userDTOList){
		for (UserDTO userDTO : userDTOList) {
			Assert.notBlank(userDTO.getUsername(),"用户名不能为空");
		}
		return R.ok(userService.addSpecialUserInfo(userDTOList),"数据成功");
	}

	/**
	 * 员工列表查询
	 * @param staff 鉴定中心员工, 这是用户名的扩充表
	 * @return
	 */
	@ApiOperation(value = "列表查询", notes = "列表查询")
	@GetMapping("/staff/list" )
	public R<List<Staff>> getStaffList(Staff staff) {

		List<Staff> staffs = staffService.getStaffVOList(Wrappers.<Staff>query()
				.like(StrUtil.isNotBlank(staff.getName()), "u.name", staff.getName())
				.orderByAsc("u.name")
		);
		return R.ok(staffs);
	}

}
