package digital.laboratory.platform.sys.controller;

import digital.laboratory.platform.sys.dto.UserDTO;
import digital.laboratory.platform.sys.service.SysOrgService;
import digital.laboratory.platform.sys.service.SysUserService;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.security.util.SecurityUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * 当前用户相关的功能
 *
 * @author zhangxl
 * @date 2021-12-28
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/currentuser")
@Api(tags = "03-当前用户管理")
public class CurrentUserController {

	private final SysUserService userService;
	private final SysOrgService sysOrgService;

	/**
	 * 获取当前用户全部信息
	 * @return 用户信息
	 */
	//@GetMapping(value = { "/detail" })
	@GetMapping
	@ApiOperation("获取当前用户全部信息")
	public R getUserInfo() {
		String userId = SecurityUtils.getUser().getId();
		return R.ok(userService.getUserVoById(userId));
	}

//	/**
//	 * 返回当前用户可代表的机构树
//	 * @return 树形机构
//	 */
//	@GetMapping(value = "/user-can-represent-tree")
//	public R getCurrentUserCanRepresentOrgTree() {
//		return R.ok(sysOrgService.getCurrentUserCanRepresentOrgTree());
//	}

	/**
	 * 修改个人信息
	 * @param userDto userDto
	 * @return success/false
	 */
	@SysLog("修改个人信息")
	@PutMapping("/edit")
	public R putCurrentUserInfoUpdate(@Valid @RequestBody UserDTO userDto) {
		String userId = SecurityUtils.getUser().getId();
		if (! userId.equals(userDto.getUserId())) {
			return R.failed("只能修改当前用户的信息");
		}
		return R.ok(userService.updateUserInfo(userDto));
	}

}
