
package digital.laboratory.platform.sys.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.common.security.annotation.Inner;
import digital.laboratory.platform.sys.entity.Area;
import digital.laboratory.platform.sys.service.SysOrgService;
import digital.laboratory.platform.sys.service.SysUserService;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import digital.laboratory.platform.sys.entity.SysOrg;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.sys.vo.AreaVO;
import digital.laboratory.platform.sys.vo.OrgVO;
import io.swagger.annotations.Api;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.security.Principal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 机构管理 前端控制器
 * </p>
 *
 * @author lengleng
 * @since 2019/2/1
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/sys/org")
@Api(tags = "11-机构管理", description="机构管理")
public class OrgController {

	private final SysOrgService sysOrgService;

	/**
	 * 通过ID查询
	 *
	 * @param id ID
	 *
	 * @return SysOrg
	 */
	@GetMapping("/{id}")
//	@Inner(value = false)
	public R getById(@PathVariable String id) {
		return R.ok(sysOrgService.getById(id));
	}

	/**
	 * 通过ID查询
	 *
	 * @param id ID
	 *
	 * @return SysOrg
	 */
	@GetMapping("/without_token/{id}")
	@Inner(value = false)
	public R getByIdWithoutToken(@PathVariable String id) {
		return R.ok(sysOrgService.getById(id));
	}

	/**
	 * 根据机构名查询机构信息
	 *
	 * @param orgname 机构名
	 *
	 * @return
	 */
	@GetMapping("/details")
	public R getByName(String orgname,String parentID) {
		SysOrg condition = new SysOrg();
		condition.setName(orgname);
		condition.setParentId(parentID);
		return R.ok(sysOrgService.getOne(new QueryWrapper<>(condition)));
	}

	/**
	 * 返回机构树的全部,  这个应该数据量很大, 应该会很少使用
	 *
	 * @return 机构树
	 */
	@GetMapping(value = "/tree")
	public R getOrgTree() {
		return R.ok(sysOrgService.listAllOrgTree());
	}

	/**
	 * 返回子机构列表, 不含隔代的后裔
	 *
	 * @return 机构树
	 */
	@GetMapping(value = "/sub_org/{orgId}")
	public R getSubOrg(@PathVariable String orgId) {
		return R.ok(sysOrgService.getSubOrg(orgId));
//		SysOrg condition = new SysOrg();
//		condition.setParentId(orgId);
//		return R.ok(sysOrgService.listObjs(new QueryWrapper<>(condition)));
	}

	/**
	 * 返回指定机构的后裔树, 不含该机构本身
	 *
	 * @return 机构树
	 */
	@GetMapping(value = "/descendants/{orgId}")
	public R getDescendantTree(@PathVariable String orgId) {
		return R.ok(sysOrgService.listDescendantTree(orgId));
	}

	/**
	 * 全部后裔的 Id 列表, 不含该机构本身
	 *
	 * @return 机构树
	 */
	@GetMapping(value = "/descendants_list/{orgId}")
	public R getDescendantsId(@PathVariable String orgId) {
		return R.ok(sysOrgService.getDescendantList(orgId));
	}


//	/**
//	 * 查收子级id列表
//	 * @return 返回子级id列表
//	 */
//	@Inner
//	@GetMapping(value = "/child-id/{orgId}")
//	public R<List<String>> listChildOrgId(@PathVariable String orgId) {
//		return R.ok(sysOrgService.listDescendantsId(orgId));
//	}

//	/**
//	 * 返回用户可代表的机构树
//	 * @return 树形机构
//	 */
//	@GetMapping(value = "/user-can-represent-tree/{id}")
//	public R getAncestorsAndDescendantsOrgTree(@PathVariable String id) {
//		if ((id != null) && (! id.isEmpty())) {
//			String orgId = userService.getUserVoById(id).getOrgId();
//			return R.ok(sysOrgService.getAncestorsAndDescendantsOrgTree(orgId));
//		}
//		else {
//			return R.failed("必须提供正确的 orgId");
//		}
//	}

	/**
	 * 返回当前用户机构树, 多根树, 去除用户所属机构后的各个分支
	 *
	 * @return 树形机构
	 */
	@GetMapping(value = "/user-trees")
	public R listCurrentUserOrgTrees() {
		return R.ok(sysOrgService.listCurrentUserOrgTrees());
	}

	/**
	 * 返回当前用户机构树, 单根树, 树为用户所属机构
	 *
	 * @return 树形机构
	 */
	@GetMapping(value = "/user-tree")
	public R listCurrentUserOrgTree() {
		return R.ok(sysOrgService.listCurrentUserOrgTree());
	}

	/**
	 * 返回当前用户机构树, 单根树, 树为用户所属机构
	 *
	 * @return 树形机构
	 */
	@GetMapping(value = "/listOrgTreeByOrgId")
	public R listOrgTreeByOrgId(String orgId) {
		return R.ok(sysOrgService.listOrgTreeByOrgId(orgId));
	}

	/**
	 * 查询机构列表
	 * 查询条件: name/alias/orgId/parentId
	 *
	 * @return 列表
	 */
	@GetMapping(value = "/vo")
	public R<List<OrgVO>> listAreaVO(SysOrg org) {
		return R.ok(sysOrgService.listAreaVO(Wrappers.<SysOrg>query()
				.like(StrUtil.isNotBlank(org.getName()), "name", org.getName())
				.like(StrUtil.isNotBlank(org.getAlias()), "alias", org.getAlias())
				.eq(StrUtil.isNotBlank(org.getOrgId()), "org_id", org.getOrgId())
				.eq(StrUtil.isNotBlank(org.getParentId()), "parent_id", org.getParentId())
		));
	}


//	/**
//	 * 根据用户userId 和 权限(permission) 查询对应的机构
//	 * 用户拥有一些机构岗位, 这些岗位有不同的权限。我们反查用户对哪些机构拥有指定的权限
//	 * 结果会是一些机构的 orgId (依赖用户是否代表对应机构的下属机构, 可能包含子机构)
//	 * 对于多个岗位, 只返回一个岗位的机构树
//	 * @param userId 用户
//	 * @param permission 权限
//	 * @return 机构树
//	 */
//	@GetMapping("/representOrgTree")
//	public R getRepresentOrgTreeByUserRolePermission(String userId, String permission) {
//		return R.ok(sysOrgService.listUserRepresentOrgTreesByPermission(userId, permission));
//	}

//	/**
//	 * 根据用户userId 和 权限(permission) 查询对应的机构
//	 * 用户拥有一些机构岗位, 这些岗位有不同的权限。我们反查用户对哪些机构拥有指定的权限
//	 * 结果会是一些机构的 orgId (依赖用户是否代表对应机构的下属机构, 可能包含子机构)
//	 * 返回全部机构列表
//	 * @param userId 用户
//	 * @param permission 权限
//	 * @return 机构列表
//	 */
//	@GetMapping("/representOrgs")
//	public R getRepresentOrgsByUserRolePermission(String userId, String permission) {
//		return R.ok(sysOrgService.listUserRepresentOrgsByPermission(userId, permission));
//	}


	/**
	 * 添加
	 *
	 * @param sysOrg 实体
	 *
	 * @return success/false
	 */
	@SysLog("添加机构")
	@PostMapping
	@PreAuthorize("@pms.hasPermission('sys_org_add')")
	public R addOrg(@Valid @RequestBody SysOrg sysOrg) {

		return R.ok(sysOrgService.addOrg(sysOrg));
	}

	/**
	 * 删除
	 *
	 * @param id ID
	 *
	 * @return success/false
	 */
	@SysLog("删除机构")
	@DeleteMapping("/{id}")
	@PreAuthorize("@pms.hasPermission('sys_org_del')")
	public R removeById(@PathVariable String id) {
		return R.ok(sysOrgService.removeOrgById(id));
	}

	/**
	 * 编辑
	 *
	 * @param sysOrg 实体
	 *
	 * @return success/false
	 */
	@SysLog("编辑机构")
	@PutMapping
	@PreAuthorize("@pms.hasPermission('sys_org_edit')")
	public R update(@Valid @RequestBody SysOrg sysOrg, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		sysOrg.setCreateBy(null);
		sysOrg.setCreateTime(null);
		sysOrg.setUpdateBy(dlpUser.getId());
		sysOrg.setUpdateTime(LocalDateTime.now());
		//sysOrg.setUpdateTime(LocalDateTime.now());
		// sysOrg.setUpdateTime(null);
		// sysOrg.setUpdateBy(null);

		return R.ok(sysOrgService.updateOrgById(sysOrg));
	}


	// /**
	//  * 当机构隶属关系有变更时, 修改机构及下属机构的祖先字段
	//  * 这主要是为了方便前端在修改涉及机构的字段时，提供了机构的祖先，前端容易一级一级的展示当前机构的从属关系
	//  *
	//  * @param sysOrg
	//  */
	// private void updateOrgAncestors(SysOrg sysOrg) {
	//
	// }

	/**
	 * 查询机构的祖先 id
	 *
	 * @param id ID
	 *
	 * @return ids array
	 */
	@Inner
	@GetMapping("/ancestorids/{id}")
	public R<List<String>> innerGetAncestorids(@PathVariable String id) {
		List<SysOrg> orgList = sysOrgService.list();
		List<String> ancestorids = new ArrayList<>();

		String currentOrgId = id;
		while (StrUtil.isNotBlank(currentOrgId)) {
			boolean found = false;
			for (SysOrg org : orgList) {
				if (org.getOrgId().equals(currentOrgId)) {
					ancestorids.add(0, currentOrgId);
					currentOrgId = org.getParentId();
					found = true;
					break;
				}
			}
			if (! found) {
				break;  // 前面的 for 循环没有找到, 说明出错了, 中止, 不然就会死循环了
			}
		}
System.out.println(String.format("getAncestorids(), id=%s ancestorids=%s", id, ancestorids));
		return R.ok(ancestorids);
	}


	/**
	 * 查询机构列表
	 * 查询条件: name/alias/orgId/parentId/area_code
	 *
	 * @return 列表
	 */
	@Inner
	@PostMapping(value = "/inner/list")
	public R<List<SysOrg>> innerGetOrgList(@RequestBody  SysOrg org) {
		System.out.println(String.format("innerGetOrgList, org=%s",org.toString()));

		return R.ok(sysOrgService.list(Wrappers.<SysOrg>query()
				.like(StrUtil.isNotBlank(org.getName()), "name", org.getName())
				.like(StrUtil.isNotBlank(org.getAlias()), "alias", org.getAlias())
				.eq(StrUtil.isNotBlank(org.getOrgId()), "org_id", org.getOrgId())
				.eq(StrUtil.isNotBlank(org.getParentId()), "parent_id", org.getParentId())
				.eq(StrUtil.isNotBlank(org.getAreaCode()), "area_code", org.getAreaCode())
		));
	}

	/**
	 * 检测机构名称是否存在
	 * @param orgname
	 * @return
	 */
	@PostMapping("/checkOrgExist")
	public R checkOrgExist(String orgname)
	{
		return  R.ok(sysOrgService.checkOrgNameExist(orgname));
	}

	/**
	 * 仅提供内部服务使用接口
	 * 根据机构id获取机构的地址省市
	 */
	@GetMapping("/fetchProvinceCityInfoByOrgId")
	public R<List<Area>> fetchProvinceCityInfoByOrgId(@RequestParam("orgId") String orgId) {
		List<Area> areaList = sysOrgService.fetchProvinceCityInfoByOrgId(orgId);
		return R.ok(areaList);
	}


}
