package digital.laboratory.platform.sys.controller;

import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.security.annotation.Inner;
import digital.laboratory.platform.sys.entity.PersonalIdentityVerifier;
import digital.laboratory.platform.sys.service.PersonalIdentityVerifierService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

/**
 * 智能身份识别终端配置
 *
 * @author Zhang Xiaolong created at  2023-02-21
 * @describe 智能身份识别终端配置 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/personal_identity_verifier" )
@Api(value = "personal_identity_verifier", tags = "智能身份识别终端配置管理")
public class PersonalIdentityVerifierController {

    private final PersonalIdentityVerifierService personalIdentityVerifierService;

	/**
	 * 通过id查询智能身份识别终端配置
	 * @param id id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{id}" )
	@Inner
	public R<PersonalIdentityVerifier> getById(@PathVariable("id" ) String id) {
        PersonalIdentityVerifier personalIdentityVerifier = personalIdentityVerifierService.getById(id);
		return R.restResult(personalIdentityVerifier,200,"查询成功！");
	}


}
