package digital.laboratory.platform.sys.controller;

import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import digital.laboratory.platform.sys.dto.SysRoleDTO;
import digital.laboratory.platform.sys.dto.SysRoleTypeDTO;
import digital.laboratory.platform.sys.dto.SysRoleTypeUserDTO;
import digital.laboratory.platform.sys.entity.SysUser;
import digital.laboratory.platform.sys.entity.SysUserRole;
import digital.laboratory.platform.sys.entity.SysRole;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.sys.service.*;
import digital.laboratory.platform.sys.vo.RoleVO;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import io.swagger.annotations.Api;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequiredArgsConstructor
@RequestMapping("/sys/role")
@Api(tags = "14-岗位管理")
public class RoleController {

    private final SysRoleService sysRoleService;
    private final SysRolePermissionService sysRolePermissionService;
    private final SysRoleTypeService sysRoleTypeService;

    private final SysUserRoleService sysUserRoleService;

    private final SysUserService sysUserService;
//	private final SysRoleMenuService sysRoleMenuService;

    /**
     * 通过ID查询岗位信息
     *
     * @param id ID
     * @return 岗位信息
     */
    @GetMapping("/{id}")
    public R getById(@PathVariable Integer id) {
        return R.ok(sysRoleService.getById(id));
    }

    /**
     * 添加岗位
     *
     * @param sysRole 岗位信息
     * @return success、false
     */
    @SysLog("添加岗位")
    @PostMapping
    @PreAuthorize("@pms.hasPermission('sys_role_add')")
    public R add(@Valid @RequestBody SysRole sysRole) {
//		String username = SecurityUtils.getUser().getUsername();
//		sysRole.setCreateBy(username);
//		sysRole.setUpdateBy(username);
        return R.ok(sysRoleService.save(sysRole));
    }

    /**
     * 修改岗位
     *
     * @param sysRole 岗位信息
     * @return success/false
     */
    @SysLog("修改岗位")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('sys_role_edit')")
    public R update(@Valid @RequestBody SysRole sysRole) {
        return R.ok(sysRoleService.updateById(sysRole));
    }

    /**
     * 删除岗位
     *
     * @param id
     * @return
     */
    @SysLog("删除岗位")
    @DeleteMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('sys_role_del')")
    public R removeById(@PathVariable String id) {
        return R.ok(sysRoleService.removeRoleById(id));
    }

    /**
     * 获取岗位列表
     *
     * @return 岗位列表
     */
    @GetMapping("/list")
    public R getRoleList() {
        List<SysRole> list = sysRoleService.list(Wrappers.emptyWrapper());
        //将岗位按照ID分组
        Map<String, List<SysRole>> map = list.stream().collect(Collectors.groupingBy(item -> item.getRoleId().substring(0, 2)));
        Set<String> roleIdSet = map.keySet();
        ArrayList<String> roleIdList = new ArrayList<>(roleIdSet);
        //排序
        Collections.sort(roleIdList, new Comparator<String>() {
            @Override
            public int compare(String o1, String o2) {
                return Integer.compare(Integer.parseInt(o1), Integer.parseInt(o2));
            }
        });
        ArrayList<SysRoleDTO> sysRoleDTOS = new ArrayList<>();
        //这里有个ID为index，是因为前端需要这个参数来遍历
        Integer index = 1;
        for (String roleId : roleIdList) {
            SysRoleDTO sysRoleDTO = new SysRoleDTO();
            List<SysRole> roleList = map.get(roleId);

            //将同一系统的岗位再次进行排序
            Collections.sort(roleList, new Comparator<SysRole>() {
                @Override
                public int compare(SysRole o1, SysRole o2) {
                    int roleId1 = Integer.parseInt(o1.getRoleId());
                    int roleId2 = Integer.parseInt(o2.getRoleId());
                    return Integer.compare(roleId1, roleId2);
                }
            });
            //因为管理员岗位与送检受理系统岗位都是10开头的ID，所以手动处理一下
            if (roleId.equals("10")) {
                List<SysRole> sysRoles = new ArrayList<>();
                SysRole sysRole1 = roleList.get(0);
                SysRole sysRole2 = roleList.get(1);
                sysRoles.add(sysRole1);
                sysRoles.add(sysRole2);
                roleList.remove(sysRole1);
                roleList.remove(sysRole2);
                SysRoleDTO sysRole = new SysRoleDTO("管理员岗", sysRoles, index);
                index++;
                sysRoleDTOS.add(sysRole);
            }
            sysRoleDTO.setRoleId(index);
            sysRoleDTO.setRoleList(roleList);
            switch (roleId) {
                case "10":
                    sysRoleDTO.setRoleSysTem("送检受理系统");
                    break;
                case "11":
                    sysRoleDTO.setRoleSysTem("污水任务系统");
                    break;
                case "23":
                    sysRoleDTO.setRoleSysTem("毛发任务系统");
                    break;
                case "24":
                    sysRoleDTO.setRoleSysTem("检材管理系统");
                    break;
                case "25":
                    sysRoleDTO.setRoleSysTem("检材流转系统");
                    break;
                case "26":
                    sysRoleDTO.setRoleSysTem("试剂耗材管理系统");
                    break;
                case "27":
                    break;
                case "29":
                    sysRoleDTO.setRoleSysTem("仪器设备管理系统");
                    break;
                case "30":
                    sysRoleDTO.setRoleSysTem("环境管理系统");
                    break;
                case "31":
                    break;
                case "32":
                    sysRoleDTO.setRoleSysTem("方法管理系统");
                    break;
                case "35":
                    sysRoleDTO.setRoleSysTem("档案管理系统");
                    break;
                case "37":
                    sysRoleDTO.setRoleSysTem("事件(筛查)系统");
                    break;
                case "38":
                    sysRoleDTO.setRoleSysTem("人员管理系统");
                    break;
                case "39":
                    sysRoleDTO.setRoleSysTem("鉴定文书管理系统");
                    break;
                case "40":
                    sysRoleDTO.setRoleSysTem("检验鉴定系统");
                    break;
                case "50":
                    sysRoleDTO.setRoleSysTem("综合统计管理岗");
            }
            if (StringUtils.isNotBlank(sysRoleDTO.getRoleSysTem())) {
                sysRoleDTOS.add(sysRoleDTO);
                index++;
            }
        }
        return R.ok(sysRoleDTOS);
    }

    /**
     * 根据岗位类别获取岗位列表
     *
     * @return 根据岗位类别获取岗位列表
     */
    @SysLog("根据岗位类别获取岗位列表")
    @GetMapping("/typeList/{id}")
    public R getRoletypeList(@PathVariable String id) {
        return R.ok(sysRoleService.list(Wrappers.<SysRole>query().eq("role_type", id)));
    }

    /**
     * 分配岗位类别给用户
     *
     * @return 分配岗位类别给用户
     */
    @SysLog("分配岗位类别给用户")
    @PostMapping("/typeUser")
    public R addUserRoleType(@RequestBody SysRoleTypeUserDTO sysRoleTypeUserDTO) {

        return R.ok(sysRoleService.addUserRoleType(sysRoleTypeUserDTO));
    }

    /**
     * /**
     * 分页查询岗位信息
     *
     * @param page 分页对象
     * @return 分页对象
     */
    @GetMapping("/page")
    public R getRolePage(Page page, SysRole sysRole) {
        //如果有类型的时候，应该查询出这个类型下的所有类型
        List<String> idLsit = new ArrayList<>();
        if (StringUtils.isNotBlank(sysRole.getRoleType())) {
            idLsit = sysRoleTypeService.getTreeChild(sysRole.getRoleType());
        }
        return R.ok(sysRoleService.page(page, Wrappers.<SysRole>query()
                .like(StringUtils.isNotBlank(sysRole.getRoleName()), "role_name", sysRole.getRoleName())
                //.like(StringUtils.isNotBlank(sysRole.getRoleId()),"",sysRole.getRoleId())
                .in(idLsit.size() > 0, "role_type", idLsit)));
    }

    /**
     * /**
     * 分配岗位类别
     *
     * @param sysRoleTypeDTO 传入参数
     * @return boolean;
     */
    @SysLog("分配岗位类别")
    @PutMapping("/addType")
    public R addRoleType(@RequestBody SysRoleTypeDTO sysRoleTypeDTO) {
        return R.ok(sysRoleService.addRoleType(sysRoleTypeDTO));
    }

    /**
     * /**
     * 移除岗位的岗位类别
     *
     * @param sysRoleList 岗位数组
     * @return boolean;
     */
    @SysLog("移除岗位类别")
    @DeleteMapping("/removeType")
    public R removeRoleType(@RequestBody List<SysRole> sysRoleList) {
        return R.ok(sysRoleService.removeRoleType(sysRoleList));
    }


    /**
     * 更新岗位权限
     *
     * @param roleVo 岗位对象
     * @return success、false
     */
    @SysLog("更新岗位权限")
    @PutMapping("/permission")
    @PreAuthorize("@pms.hasPermission('sys_role_perm')")
    public R saveRolePermissions(@RequestBody RoleVO roleVo) {
        SysRole sysRole = sysRoleService.getById(roleVo.getRoleId());
        return R.ok(sysRolePermissionService.saveRolePermissions(sysRole.getRoleId(), roleVo.getPermissionIds()));
    }

    /**
     * 获取岗位下的用户ID
     *
     * @param roleType
     * @return
     */
    @GetMapping("/getRoleContainsUser")
    public R getRoleContainsUser(String roleType) {
        List<SysUser> retUserList = new ArrayList<>();
        //取出这个岗位类型包含的岗位
        List<String> typeIdList = sysRoleTypeService.getTreeChild(roleType);

        List<SysRole> roleList = sysRoleService.list(Wrappers.<SysRole>query()
                .in(typeIdList.size() > 0, "role_type", typeIdList));

        List<String> roleIdList = new ArrayList<>();//将这些岗位转化为岗位ID
        roleList.forEach(role -> {
            roleIdList.add(role.getRoleId());
        });
        if (roleIdList.size() > 0) {
            List<String> userIdList = new ArrayList<>();
            //根据这些岗位查询出对应的用户
            List<SysUserRole> sysUserRoleList = sysUserRoleService.list(Wrappers.<SysUserRole>query()
                    .in("role_id", roleIdList));
            sysUserRoleList.forEach(sysUserRole -> {
                userIdList.add(sysUserRole.getUserId());
            });

            if (userIdList.size() > 0) {
                retUserList = sysUserService.list(Wrappers.<SysUser>query()
                        .in("user_id", userIdList));
                return R.ok(retUserList);
            } else {
                return R.ok(retUserList);
            }
        } else {
            return R.ok(retUserList);
        }
    }

//	/**
//	 * 更新岗位菜单
//	 * @param roleVo 岗位对象
//	 * @return success、false
//	 */
//	@SysLog("更新岗位菜单")
//	@PutMapping("/menu")
//	@PreAuthorize("@pms.hasPermission('sys_role_perm')")
//	public R saveRoleMenus(@RequestBody RoleVO roleVo) {
//		SysRole sysRole = sysRoleService.getById(roleVo.getRoleId());
//		return R.ok(sysRoleMenuService.saveRoleMenus(sysRole.getRoleCode(), roleVo.getRoleId(), roleVo.getMenuIds()));
//	}

}
