package digital.laboratory.platform.sys.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.core.util.TestUtils;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.sys.entity.PartialStaff;
import digital.laboratory.platform.sys.entity.SysUser;
import digital.laboratory.platform.sys.service.StaffService;
import digital.laboratory.platform.sys.service.SysUserService;
import digital.laboratory.platform.sys.vo.Staff;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * 鉴定中心员工, 这是用户名的扩充表
 *
 * @author Zhang Xiaolong created at  2022-04-12
 * @describe 鉴定中心员工, 这是用户名的扩充表 前端控制器
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/sys/staff" )
@Api(tags = "21-鉴定中心员工, 这是用户表的扩充")
public class StaffController {

    private final StaffService staffService;
	private final SysUserService userService;

	/**
	 * 通过id查询鉴定中心员工, 这是用户名的扩充表
	 * @param userId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{userId}" )
	@PreAuthorize("@pms.hasPermission('StaffGet')" )
	public R getById(@PathVariable("userId" ) String userId) {
		return R.ok(staffService.getById(userId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param staff 鉴定中心员工, 这是用户名的扩充表
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('StaffList')" )
    public R<IPage<Staff>> getStaffPage(Page page, Staff staff) {
		IPage<Staff> thePage = staffService.getStaffVOPage(page, Wrappers.<Staff>query()
				.like(StrUtil.isNotBlank(staff.getName()), "u.name", staff.getName())
				.like(StrUtil.isNotBlank(staff.getUsername()), "u.username", staff.getUsername())
				.eq(StrUtil.isNotBlank(staff.getPoliceNo()), "u.police_no", staff.getPoliceNo())
				.orderByAsc("u.name")
		);
		return R.ok(thePage);
    }

	/**
	 * 列表查询
	 * @param staff 鉴定中心员工, 这是用户名的扩充表
	 * @return
	 */
	@ApiOperation(value = "列表查询", notes = "列表查询")
	@GetMapping("/list" )
	@PreAuthorize("@pms.hasPermission('StaffList')" )
	public R<List<Staff>> getStaffList(Staff staff) {

		List<Staff> staffs = staffService.getStaffVOList(Wrappers.<Staff>query()
				.like(StrUtil.isNotBlank(staff.getName()), "u.name", staff.getName())
				.orderByAsc("u.name")
		);
		return R.ok(staffs);
	}


	/**
     * 新增鉴定中心员工, 这是用户名的扩充表
     * @param partialStaff 鉴定中心员工, 这是用户名的扩充表
     * @return R
     */
    @ApiOperation(value = "新增鉴定中心员工, 这是用户名的扩充表", notes = "新增鉴定中心员工, 这是用户名的扩充表")
    @SysLog("新增鉴定中心员工, 这是用户名的扩充表" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('StaffCreate')" )
    public R postAddObject(@RequestBody PartialStaff partialStaff) {
        return R.ok(staffService.save(partialStaff));
    }

    /**
     * 修改鉴定中心员工, 这是用户名的扩充表
     * @param partialStaff 鉴定中心员工, 这是用户名的扩充表
     * @return R
     */
    @ApiOperation(value = "修改鉴定中心员工, 这是用户名的扩充表", notes = "修改鉴定中心员工, 这是用户名的扩充表")
    @SysLog("修改鉴定中心员工, 这是用户名的扩充表" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('StaffEdit')" )
    public R putUpdateById(@RequestBody PartialStaff partialStaff) {
        return R.ok(staffService.updateById(partialStaff));
    }

    /**
     * 通过id删除鉴定中心员工, 这是用户名的扩充表
     * @param userId id
     * @return R
     */
    @ApiOperation(value = "通过id删除鉴定中心员工, 这是用户名的扩充表", notes = "通过id删除鉴定中心员工, 这是用户名的扩充表")
    @SysLog("通过id删除鉴定中心员工, 这是用户名的扩充表" )
    @DeleteMapping("/{userId}" )
    @PreAuthorize("@pms.hasPermission('StaffDelete')" )
    public R deleteById(@PathVariable String userId) {
        return R.ok(staffService.removeById(userId));
    }


	//=====================================================================================
	// 测试, 创建 20 个员工
	@GetMapping("/create20" )
	public R create20() {
		for (int i=0; i<20; i++)
		{
			SysUser user = new SysUser();

			user.setUserId(IdWorker.get32UUID().toUpperCase());
			user.setUsername("partialStaff"+RandomUtil.randomString(RandomUtil.randomInt(1, 6)));
			user.setEnabled(true);
			user.setPassword("$2a$10$Q/8pJPrJLzCSMup8akH3y.Nxra4H/jGdBQTNDyl70W5Rz8EatLtmO");
			user.setPoliceType(RandomUtil.randomInt(0, 2));
			user.setPoliceNo(RandomUtil.randomNumbers(8));
			user.setNativePlace(TestUtils.genAddress(i));

			//当前时间字符串, 格式: yyyy-MM-dd HH:mm:ss
			String now = DateUtil.now();
			//当前日期字符串, 格式: yyyy-MM-dd
			String today= DateUtil.today();
			user.setName("测试用户-"+today+"-"+String.format("%d", i));
			user.setFormerName("测试用户曾用名-"+today+"-"+String.format("%d", i));

			user.setGender(RandomUtil.randomInt(2));
			user.setNation(TestUtils.genNation());
			user.setMaritalStatus(TestUtils.genMaritalStatus());
			user.setMilitaryServiceStatus(TestUtils.genMilitaryServiceStatus());
			user.setEducationalLevel(TestUtils.genEducationalLevel());
			user.setBirthPlace(TestUtils.genAddress(i));
			user.setCertificateType(TestUtils.genCertificateType());
			user.setCertificateNo(RandomUtil.randomNumbers(18));
			user.setIdentificationCardNo(RandomUtil.randomNumbers(18));
			user.setMobile(TestUtils.genMobile());
			user.setEmail(RandomUtil.randomStringUpper(10)+"@"+RandomUtil.randomStringUpper(10)+".COM");
			user.setAddress(TestUtils.genAddress(i));

			user.setPosition(TestUtils.genPosition());

			userService.save(user);

			PartialStaff partialStaff = new PartialStaff();
			partialStaff.setUserId(user.getUserId());
			staffService.save(partialStaff);
		}
		return R.ok("已经创建 20 个员工");
	}
}
