package digital.laboratory.platform.sys.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.sys.entity.StaffTrainingPlan;
import digital.laboratory.platform.sys.service.StaffTrainingPlanService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;


/**
 * 员工培训计划
 *
 * @author Zhang Xiaolong created at  2022-04-12
 * @describe 员工培训计划 前端控制器
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/sys/staff_training_plan" )
@Api( tags = "员工培训计划管理")
public class StaffTrainingPlanController {

    private final StaffTrainingPlanService staffTrainingPlanService;

	/**
	 * 通过id查询员工培训计划
	 * @param id id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{id}" )
	@PreAuthorize("@pms.hasPermission('sys_staff_training_plan_get')" )
	public R getById(@PathVariable("id" ) String id) {
		return R.ok(staffTrainingPlanService.getById(id));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param staffTrainingPlan 员工培训计划
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('sys_staff_training_plan_get')" )
    public R getStaffTrainingPlanPage(Page page, StaffTrainingPlan staffTrainingPlan) {
        return R.ok(staffTrainingPlanService.page(page, Wrappers.query(staffTrainingPlan)));
    }


    /**
     * 新增员工培训计划
     * @param staffTrainingPlan 员工培训计划
     * @return R
     */
    @ApiOperation(value = "新增员工培训计划", notes = "新增员工培训计划")
    @SysLog("新增员工培训计划" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('sys_staff_training_plan_add')" )
    public R postAddObject(@RequestBody StaffTrainingPlan staffTrainingPlan) {
        return R.ok(staffTrainingPlanService.save(staffTrainingPlan));
    }

    /**
     * 修改员工培训计划
     * @param staffTrainingPlan 员工培训计划
     * @return R
     */
    @ApiOperation(value = "修改员工培训计划", notes = "修改员工培训计划")
    @SysLog("修改员工培训计划" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('sys_staff_training_plan_edit')" )
    public R putUpdateById(@RequestBody StaffTrainingPlan staffTrainingPlan) {
        return R.ok(staffTrainingPlanService.updateById(staffTrainingPlan));
    }

    /**
     * 通过id删除员工培训计划
     * @param id id
     * @return R
     */
    @ApiOperation(value = "通过id删除员工培训计划", notes = "通过id删除员工培训计划")
    @SysLog("通过id删除员工培训计划" )
    @DeleteMapping("/{id}" )
    @PreAuthorize("@pms.hasPermission('sys_staff_training_plan_del')" )
    public R deleteById(@PathVariable String id) {
        return R.ok(staffTrainingPlanService.removeById(id));
    }

}
