package digital.laboratory.platform.sys.controller;

import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sys.entity.SysRoleType;
import digital.laboratory.platform.sys.service.SysRoleTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;

@RestController
@RequiredArgsConstructor
@RequestMapping("/sys/roletype")
@Api(value = "RoleType", tags = "岗位类别管理")
public class SysRoleTypeController {
    private final SysRoleTypeService sysRoleTypeService;
    /**
     * 新增岗位类别
     *
     * @param sysRoleType
     * @return R
     */
    @ApiOperation(value = "新增岗位类别", notes = "新增岗位类别")
    @PostMapping()
//    @PreAuthorize("@pms.hasPermission('sys_role_type_add')")
    public R addRoleType(@RequestBody SysRoleType sysRoleType, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        return R.ok(sysRoleTypeService.addRoleType(sysRoleType));
    }

    /**
     * 修改岗位类别
     *
     * @param sysRoleType
     * @return R
     */
    @ApiOperation(value = "修改岗位类别", notes = "修改岗位类别")
    @PutMapping()
//    @PreAuthorize("@pms.hasPermission('sys_role_type_edit')")
    public R editRoleType(@RequestBody SysRoleType sysRoleType, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        if (sysRoleTypeService.updateById(sysRoleType)) {
            return R.ok(sysRoleType);
        } else return R.failed(sysRoleType, "修改失败");
    }

    /**
     * 删除岗位类别
     *
     * @param id
     * @return R
     */
    @ApiOperation(value = "删除岗位类别", notes = "删除岗位类别")
    @DeleteMapping("/{id}")
//    @PreAuthorize("@pms.hasPermission('sys_role_type_del')")
    public R delRoleType(@PathVariable String  id, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        if (sysRoleTypeService.delSysRoleType(id)) {
            return R.ok("删除成功");
        } else return R.failed("删除失败");
    }

    /**
     * 通过ID查询岗位类别
     *
     * @param id
     * @return R
     */
    @ApiOperation(value = "通过ID查询岗位类别", notes = "通过ID查询岗位类别")
    @GetMapping("/{id}")
//    @PreAuthorize("@pms.hasPermission('sys_role_type_get)")
    public R getRoleType(@PathVariable String id, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        return R.ok(sysRoleTypeService.getSysRoleType(id));

    }

    /**
     * 返回树形字典集合
     *
     * @param parentId 父节点ID
     * @return 树形权限
     */
    @ApiOperation(value = "取岗位类别分支树", notes = "取岗位类别分支树")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "parentId", value = "分支树的标识", required = true, paramType = "query"),
    })

    @GetMapping("/tree")
//    @PreAuthorize("@pms.hasPermission('sys_role_type_get')")
    public R treeSysRoleType(String parentId, HttpServletRequest theHttpServletRequest) {
        return R.ok(sysRoleTypeService.treeSysRoleType(parentId));
    }

    @GetMapping(value = "/treeParentData")
    //    @PreAuthorize("@pms.hasPermission('sys_role_type_get')")
    public R getTreeParentSysRoleType(String parentId) {
        return R.ok(sysRoleTypeService.getTreeParentSysRoleType());
    }

}
