package digital.laboratory.platform.sys.controller;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import digital.laboratory.platform.common.core.util.TestUtils;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.sys.entity.PartialStaff;
import digital.laboratory.platform.sys.entity.SysOrg;
import digital.laboratory.platform.sys.entity.SysRole;
import digital.laboratory.platform.sys.service.StaffService;
import digital.laboratory.platform.sys.service.SysOrgService;
import digital.laboratory.platform.sys.service.SysUserService;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import digital.laboratory.platform.sys.dto.UserDTO;
import digital.laboratory.platform.sys.dto.UserInfo;
import digital.laboratory.platform.sys.entity.SysUser;
import digital.laboratory.platform.sys.vo.Staff;
import digital.laboratory.platform.sys.vo.UserInfoVO;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.security.annotation.Inner;
import digital.laboratory.platform.common.security.util.SecurityUtils;
import io.swagger.annotations.*;
import lombok.RequiredArgsConstructor;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.security.Principal;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * @author lengleng
 * @date 2019/2/1
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/sys/user")
@Api(tags = "12-用户管理", description = "用户管理, 通常需要有管理员身份才能进行的操作")
public class UserController {

    private final SysUserService userService;
    private final SysOrgService sysOrgService;
    private final StaffService staffService;

    /**
     * 获取当前用户全部信息
     *
     * @return 用户信息
     */
    @GetMapping(value = {"/info"})
    @ApiOperation("获取当前用户全部信息")
    public R getUserInfo(HttpServletRequest request) {

//System.out.println(String.format("getRemoteAddr=%s getRemoteHost=%s getRemotePort=%d", request.getRemoteAddr(), request.getRemoteHost(), request.getRemotePort()));
        String username = SecurityUtils.getUser().getUsername();
        SysUser user = userService.getOne(Wrappers.<SysUser>query().lambda().eq(SysUser::getUsername, username));
        //SysUser user = userService.getUserByUsername(username);

        if (user == null) {
            return R.failed("获取当前用户信息失败");
        }
        UserInfo userInfo = userService.getUserInfo(user);
        UserInfoVO vo = new UserInfoVO();
        vo.setSysUser(userInfo.getSysUser());
        List<SysRole> roleList = userInfo.getRoleList();
        vo.setRoleList(roleList);
        vo.setRoles(userInfo.getRoles());
        vo.setPermissions(userInfo.getPermissions());
        vo.setOrg(userInfo.getOrg());
        return R.ok(vo);
    }


    /**
     * 获取指定用户全部信息
     *
     * @return 用户信息
     */
    @Inner
    @GetMapping("/info/{username}")
    public R innerGetUserInfoByUsername(@PathVariable String username) {
        SysUser user = userService.getOne(Wrappers.<SysUser>query().lambda().eq(SysUser::getUsername, username));
        //SysUser user = userService.getUserByUsername(username);
        if (user == null) {
            return R.failed(String.format("用户信息为空 %s", username));
        }
        return R.ok(userService.getUserInfo(user));
    }

    /**
     * 获取当前用户管理的全部用户 id
     *
     * @return 用户 id 集合
     */
    @GetMapping("/manageids")
    public R<List<String>> getManageUserIds() {
        //return R.ok(userService.listUserIdByOrgIds(orgIds));
        Page page = new Page();
        UserDTO userDTO = new UserDTO();
        IPage ipage = userService.getUserWithRolePage(page, userDTO);

        return R.ok(ipage.getRecords());
    }


    /**
     * 根据机构id, 查询对应的用户 id 集合
     *
     * @param orgIds 机构id 集合
     * @return 用户 id 集合
     */
    @Inner
    @GetMapping("/ids")
    public R<List<String>> innerGetUserIdsByOrgIds(@RequestParam("orgIds") Set<String> orgIds) {
        return R.ok(userService.listUserIdByOrgIds(orgIds));
    }

    /**
     * 通过ID查询用户信息
     *
     * @param id ID
     * @return 用户信息
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "根据 userId 取用户详细信息", notes = "返回的用户信息中有详细的用户信息, 除了对应的用户表中该用户所有字段信息之外, 还提供了机构名称(orgName)、用户岗位列表(userRoleList)。\n" +
            "userRoleList是一个数组, 每一个元素包含userId、roleId、representOrgId、representOrgName、representSubOrg等, 还包含对应岗位的完整信息。\n" +
            "这个接口提供的数据尽量保证完整性, 调用者得到的数据是完整的, 不需要再次调用其他的接口去获取对应数据的进一步信息。")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "要查询的用户的 userId", required = true, dataType = "string", example = "1", paramType = "query")
    })
    public R getById(@PathVariable String id) {
//		Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
//		if (authentication != null) {
//			Collection<? extends GrantedAuthority> authorities = authentication.getAuthorities();
//			String ppp = authorities.stream().map(GrantedAuthority::getAuthority)
//					.filter(StringUtils::hasText)
//					.filter(x -> PatternMatchUtils.simpleMatch(permission, x))
//					.findAny().get();
//		}
///////////
        return R.ok(userService.getUserVoById(id));
    }

    /**
     * 根据用户名查询用户信息
     *
     * @param username 用户名
     * @return 用户信息
     */
    @GetMapping("/brief/{username}")
    public R getUserBriefByUsername(@PathVariable String username) {
//		SysUser condition = new SysUser();
//		condition.setUsername(username);
//		return R.ok(userService.getOne(new QueryWrapper<>(condition)));

        SysUser user = userService.getOne(Wrappers.<SysUser>query().lambda().eq(SysUser::getUsername, username));
        //SysUser user = userService.getUserByUsername(username);
        return R.ok(user);

    }

    /**
     * 删除用户信息
     *
     * @param id ID
     * @return R
     */
    @SysLog("删除用户信息")
    @DeleteMapping("/{id}")
    @PreAuthorize("@pms.hasPermission('sys_user_del')")
    public R deleteUserById(@PathVariable String id) {
        SysUser sysUser = userService.getById(id);
        return R.ok(userService.removeUserById(sysUser));
    }

    /**
     * 添加用户
     *
     * @param userDto 用户信息
     * @return success/false
     */
    @SysLog("添加用户")
    @PostMapping
    @PreAuthorize("@pms.hasPermission('sys_user_add')")
    public R postUserAdd(@RequestBody UserDTO userDto, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        System.out.println(String.format("添加新用户, username=%s userId=%s", userDto.getUsername(), userDto.getUserId()));
        List<SysOrg> orgList = sysOrgService.getDescendantList(dlpUser.getOrgId());
        //如果添加的用户和当前用户的机构ID相同，则直接添加进去，否则的话，需要判断是否是当前用户机构的子机构
        if (StrUtil.equalsIgnoreCase(dlpUser.getOrgId(), userDto.getOrgId())) {
            return R.ok(userService.addUser(userDto));
        }
        for (SysOrg org : orgList) {
            if (StrUtil.equalsIgnoreCase(org.getOrgId(), userDto.getOrgId())) {
                // 新增用户的机构 是 当前用户的下属机构
                return R.ok(userService.addUser(userDto));
            }
        }
        throw new RuntimeException("用户所属机构只能是你所属机构，或你所属机构的下级机构");
    }

    /**
     * 更新用户信息
     *
     * @param userDto 用户信息
     * @return R
     */
    @SysLog("更新用户信息")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('sys_user_edit')")
    public R putUserUpdate(@Valid @RequestBody UserDTO userDto, HttpServletRequest theHttpServletRequest) {
        Principal principal = theHttpServletRequest.getUserPrincipal();
        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        System.out.println(String.format("更新用户信息, username=%s userId=%s avatar=%s", userDto.getUsername(), userDto.getUserId(), userDto.getAvatar()));
        List<SysOrg> orgList = sysOrgService.getDescendantList(dlpUser.getOrgId());
        //如果添加的用户和当前用户的机构ID相同，则直接更新，否则的话，需要判断是否是当前用户机构的子机构
        if (StrUtil.equals(dlpUser.getOrgId(), userDto.getOrgId())) {
            return R.ok(userService.updateUser(userDto));
        }
        for (SysOrg org : orgList) {
            if (StrUtil.equalsIgnoreCase(org.getOrgId(), userDto.getOrgId())) {
                // 待修改用户的机构 是 当前用户的下属机构
                return R.ok(userService.updateUser(userDto));
            }
        }
        throw new RuntimeException("用户所属机构只能是你所属机构，或你所属机构的下级机构");
    }

    /**
     * 分页查询用户
     *
     * @param page    参数集
     * @param userDTO 查询参数列表
     * @return 用户集合
     */
    @GetMapping("/page")
    public R getUserPage(Page page, UserDTO userDTO) {
        return R.ok(userService.getUserWithRolePage(page, userDTO));
    }

    /**
     * 查询用户列表
     *
     * @return 用户集合
     */
    @GetMapping("/list")
    public R getUserList() {
        return R.ok(userService.list());
    }

    /**
     * 按 org_id 查用户列表
     *
     * @param orgId 机构标识
     * @return 用户集合
     */
    @ApiOperation(value = "按 org_id 查用户列表", notes = "按 org_id 查用户列表\n" +
            "使用场景是: 创建员工时, 需要选择机构为分中心的用户。\n"
    )
    @GetMapping("/list_by_org")
    public R getUserListByOrg(String orgId) {
        if (StrUtil.isBlank(orgId)) {
            throw new RuntimeException("必须提供 orgId 作为参数");
        }
        //创建员工时，获取的用户必须是没有被添加成员工的数据，所以此处需要过滤一下
        List<PartialStaff> staffList = staffService.list();
        List<String> staffUserIds = new ArrayList<String>();
        for (PartialStaff partialStaffObj : staffList) {
            staffUserIds.add(partialStaffObj.getUserId());
        }
        List<SysUser> userList = userService.list(Wrappers.<SysUser>query()
                .eq(StrUtil.isNotBlank(orgId), "org_id", orgId)
                .notIn(staffList != null, "user_id", staffUserIds)
                .orderByAsc("name")
        );
        return R.ok(userList);
    }


    @GetMapping("/staff_list")
    public R<List<Staff>> getStaffList(@RequestParam("name") String name) {

        System.out.println(String.format("getStaffList(), name=%s", name));

        List<Staff> staffs = userService.getStaffList(Wrappers.<Staff>query()
                .like(StrUtil.isNotBlank(name), "u.name", name)
                .orderByAsc("u.name")
        );
        return R.ok(staffs);
    }


    /**
     * 查询对指定的机构拥有某些权限的用户列表。
     * <p>
     * 还可以通过 includeSubOrg, 对用户进行筛选。
     * includeSubOrg 为 true 时, 如果指定机构是某用户所属机构的下属机构时该用户有效;
     * includeSubOrg 为 false 时, 只有当指定机构是某用户所属机构时该用户有效;
     *
     * @param orgId       机构 id
     *                    param includeSubOrg 是否包含子机构
     * @param permissions 权限集合
     * @return 用户列表
     */
    @GetMapping("/bypermissions")
    public R<List<SysUser>> GetUsersByPermission(@RequestParam("orgId") String orgId, @RequestParam("permissions") List<String> permissions) {
        //rolePermissionService
        return R.ok(userService.listUsersByPermission(orgId, false, permissions));
    }


    /**
     * 查询对指定的机构拥有某些权限的用户列表。
     * <p>
     * 还可以通过 includeSubOrg, 对用户进行筛选。
     * includeSubOrg 为 true 时, 如果指定机构是某用户所属机构的下属机构时该用户有效;
     * includeSubOrg 为 false 时, 只有当指定机构是某用户所属机构时该用户有效;
     *
     * @param orgId       机构 id
     *                    param includeSubOrg 是否包含子机构
     * @param permissions 权限集合
     * @return 用户列表
     */
    @Inner
    @GetMapping("/inner_bypermissions")
    public R<List<SysUser>> innerGetUsersByPermission(@RequestParam("orgId") String orgId, @RequestParam("permissions") List<String> permissions) {
        return R.ok(userService.listUsersByPermission(orgId, false, permissions));
    }

    /**
     * 审核注册的用户，注册用户是需要管理员审核才能启用
     *
     * @param userId
     * @param op_code
     * @return
     */
    @ApiOperation(value = "审核用户", notes = "新注册的用户需要审核才可使用\n")
    @PutMapping("/reviewUser")
    public R reviewRegisterUser(@RequestParam("userId") String userId, @RequestParam("op_code") boolean op_code) throws Exception {
        if (StrUtil.isBlank(userId) || userId == null) {
            throw new Exception("用户ID不能为空");
        }
        return R.ok(userService.reviewUser(userId, op_code));
    }

    /**
     * 修改个人信息
     *
     * @param userDto userDto
     * @return success/false
     */
    @SysLog("修改个人信息")
    @PutMapping("/edit")
    public R putCurrentUserInfoUpdate(@Valid @RequestBody UserDTO userDto) {
        return R.ok(userService.updateUserInfo(userDto));
    }

    /**
     * @param username 用户名称
     * @return 上级机构用户列表
     */
    @GetMapping("/ancestor/{username}")
    public R getAncestorUserListByUsername(@PathVariable String username) {
        return R.ok(userService.listAncestorUsersByUsername(username));
    }

    //==================================================================================================================
    // 以下是管理员使用的接口
    // 这些提供给管理员使用, 处理的数据不是管理员这个用户本身的数据, 而是其他用户的数据
    //==================================================================================================================

    //==================================================================================================================
    // 以下是内部使用的接口
    //==================================================================================================================

    /**
     * 通过 ID 查询用户信息
     *
     * @param id ID
     * @return 用户信息
     */
    @Inner
    @GetMapping("/inner_get_by_id/{id}")
    public R<SysUser> innerGetById(@PathVariable String id) {
        return R.ok(userService.getById(id));
    }

    /**
     * 根据部门名称和是否是部门领导查询用户信息
     *
     * @param department
     * @param deptHeader
     * @return
     */
    @GetMapping("/by_department_and_dept_header")
    public R<List<SysUser>> getUserInfoByDepartmentAndDeptHeader(@RequestParam("department") String department,
                                                                 @RequestParam("deptHeader") Integer deptHeader) {

        if (StrUtil.isBlank(department)) {
            throw new RuntimeException("部门不能为空");
        }

        if (deptHeader == null) {
            throw new RuntimeException("是否部门领导不能为空");
        }

        List<SysUser> sysUserList = userService.list(
                Wrappers.<SysUser>query()
                        .eq("department", department.trim()) // 防止参数传递过来时有空格
                        .eq("dept_header", deptHeader)); // 是否部门领导: 0=不是, 1=正职领导, 2=副职领导

        return R.ok(sysUserList);
    }


    //==================================================================================================================
    // 以下是测试使用的接口
    //==================================================================================================================

    // 测试, 创建 100 个案件
    @GetMapping("/create100")
    public R xxx_TestCreate100() {
        for (int i = 0; i < 100; i++) {
            SysUser user = new SysUser();

            user.setUserId(IdWorker.get32UUID().toUpperCase());
            user.setUsername(RandomUtil.randomString(RandomUtil.randomInt(4, 10)));
            user.setEnabled(true);
            user.setPassword("$2a$10$Q/8pJPrJLzCSMup8akH3y.Nxra4H/jGdBQTNDyl70W5Rz8EatLtmO");
            user.setPoliceType(RandomUtil.randomInt(0, 2));
            user.setPoliceNo(RandomUtil.randomNumbers(8));
            user.setNativePlace(TestUtils.genAddress(i));

            //当前时间字符串, 格式: yyyy-MM-dd HH:mm:ss
            String now = DateUtil.now();
            //当前日期字符串, 格式: yyyy-MM-dd
            String today = DateUtil.today();
            user.setName("测试用户-" + today + "-" + String.format("%d", i));
            user.setFormerName("测试用户曾用名-" + today + "-" + String.format("%d", i));

            user.setGender(RandomUtil.randomInt(2));
            user.setNation(TestUtils.genNation());
            user.setMaritalStatus(TestUtils.genMaritalStatus());
            user.setMilitaryServiceStatus(TestUtils.genMilitaryServiceStatus());
            user.setEducationalLevel(TestUtils.genEducationalLevel());
            user.setBirthPlace(TestUtils.genAddress(i));
            user.setCertificateType(TestUtils.genCertificateType());
            user.setCertificateNo(RandomUtil.randomNumbers(18));
            user.setIdentificationCardNo(RandomUtil.randomNumbers(18));
            user.setMobile(TestUtils.genMobile());
            user.setEmail(RandomUtil.randomStringUpper(10) + "@" + RandomUtil.randomStringUpper(10) + ".COM");
            user.setAddress(TestUtils.genAddress(i));

            user.setPosition(TestUtils.genPosition());

            userService.save(user);
        }
        return R.ok("已经创建 100 个用户");
    }

    @DeleteMapping("/del/signature/{userId}")
    @ApiOperation(value = "删除电子签名",notes = "通过传入用户ID，直接删除该用户电子签名数据")
    public R deleteSignature(@PathVariable String userId) {
        SysUser user = userService.getById(userId);
        if (user == null) {
            throw new RuntimeException(String.format("未能查到ID为:" + userId + "的用户信息"));
        }
        user.setSignature("");
        return userService.updateById(user) ? R.ok("删除成功") : R.failed("删除失败");
    }

}
