package digital.laboratory.platform.sys.service;

import cn.hutool.core.lang.tree.Tree;
import com.baomidou.mybatisplus.extension.service.IService;
import digital.laboratory.platform.sys.entity.SysMenu;

import java.util.List;
import java.util.Set;

/**
 * <p>
 * 菜单权限表 服务类
 * </p>
 *
 * @author lengleng
 * @since 2019/2/1
 */
public interface SysMenuService extends IService<SysMenu> {

	/**
	 * 通过岗位编号查询URL 权限
	 * @param roleIds 岗位ID
	 * @return 菜单列表
	 */
	Set<SysMenu> findMenuByRoleIds(List<String> roleIds);

	Set<SysMenu> findMenuByUserId(String userId);

	/**
	 * 级联删除菜单
	 * @param id 菜单ID
	 * @return true成功, false失败
	 */
	Boolean removeMenuById(String id);

	/**
	 * 更新菜单信息
	 * @param sysMenu 菜单信息
	 * @return 成功、失败
	 */
	Boolean updateMenuById(SysMenu sysMenu);

	/**
	 * 根据菜单标识构建树
	 * @param menu 菜单标识
	 * @return 菜单树
	 */
	List<Tree<String>> treeMenuByMenu(String menu);

	/**
	 * 构建树
	 * @param lazy 是否是懒加载
	 * @param parentId 父节点ID
	 * @return 菜单树
	 */
	List<Tree<String>> treeMenuByParentId(boolean lazy, String parentId);

	/**
	 * 查询菜单
	 * @param menuSet
	 * @param parentId
	 * @return 菜单树
	 */
	List<Tree<String>> filterMenu(Set<SysMenu> menuSet, String parentId);

}
