package digital.laboratory.platform.sys.service;

import cn.hutool.core.lang.tree.Tree;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import digital.laboratory.platform.sys.entity.Area;
import digital.laboratory.platform.sys.entity.SysOrg;
import digital.laboratory.platform.sys.vo.OrgVO;

import java.util.List;

/**
 * <p>
 * 机构管理 服务类
 * </p>
 *
 * @author lengleng
 * @since 2019/2/1
 */
public interface SysOrgService extends IService<SysOrg> {

	/**
	 * 查询机构树
	 * @return 树
	 */
	List<Tree<String>> listAllOrgTree();

	/**
	 * 查询用户机构树, 单根树, 树为用户所属机构
	 * @return
	 */
	Tree<String> listCurrentUserOrgTree();

	/**
	 * 查询用户机构树, 多根树, 去除用户所属机构后的各个分支
	 * @return
	 */
	List<Tree<String>> listCurrentUserOrgTrees();

	/**
	 * 返回子机构列表, 不含隔代的后裔
	 * @param orgId 机构id
	 * @return List<String>
	 */
	List<SysOrg> getSubOrg(String orgId);

	/**
	 * 列举指定机构的祖先和后裔树
	 * 从指定节点开始, 向下是所属的全部机构, 向上是其父节点, 一直回溯到根
	 * @param orgId
	 * @return
	 */
	public List<Tree<String>> getAncestorsAndDescendantsOrgTree(String orgId);	// 列举指定机构的祖先和后裔树

	/**
	 * 查询当前用户可代表的机构树
	 * @return
	*/
	List<Tree<String>> getCurrentUserCanRepresentOrgTree();

	/**
	 * 添加信息机构
	 * @param sysOrg
	 * @return
	 */
	Boolean addOrg(SysOrg sysOrg);

	/**
	 * 删除机构
	 * @param id 机构 ID
	 * @return 成功、失败
	 */
	Boolean removeOrgById(String id);

	/**
	 * 更新机构
	 * @param sysOrg 机构信息
	 * @return 成功、失败
	 */
	Boolean updateOrgById(SysOrg sysOrg);

	/**
	 * 查找指定机构的子机构id列表
	 * @param orgId 机构id
	 * @return List<String>
	 */
	List<SysOrg> getDescendantList(String orgId);

	/**
	 * 查找指定机构的子机构列表
	 * @param orgId 机构id
	 * @return List<SysOrg>
	 */
	public List<Tree<String>> listDescendantTree(String orgId);

	List<OrgVO> listAreaVO(QueryWrapper<SysOrg> qw);

	/**
	 * 检查机构是否已经存在，如果存在返回true，不存在返回false
	 * @param orgName
	 * @return
	 */
	public boolean checkOrgNameExist(String orgName);

	/**
	 * 根据机构id返回机构树
	 * @param orgId
	 * @return
	 */
	Tree<String> listOrgTreeByOrgId(String orgId);

	/**
	 * 仅提供内部服务使用接口
	 * 根据机构id获取机构的地址省市
	 */
    List<Area> fetchProvinceCityInfoByOrgId(String orgId);
}
