package digital.laboratory.platform.sys.service;

import digital.laboratory.platform.sys.entity.SysUser;
import digital.laboratory.platform.sys.mapper.SysOrgMapper;
import digital.laboratory.platform.sys.mapper.SysUserMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor
public class SysUserQuery {
	private final SysOrgMapper sysOrgMapper;
	private final SysOrgService sysOrgService;
	private final SysUserService userService;
	private final SysUserMapper userMapper;

	/**
	 * 查询对指定的机构拥有某些权限的用户列表。
	 *
	 * 还可以通过 includeSubOrg, 对用户进行筛选。
	 * includeSubOrg 为 true 时, 如果指定机构是某用户所属机构的下属机构时该用户有效;
	 * includeSubOrg 为 false 时, 只有当指定机构是某用户所属机构时该用户有效;
	 *
	 * @param orgId 机构 id
	 * @param includeSubOrg 是否包含子机构
	 * @param permissions 权限集合
	 *
	 * @return 用户列表
	 */
	public List<SysUser> listUsersByPermission(String orgId, boolean includeSubOrg, List<String> permissions) {
		System.out.println(String.format("public List<SysUser> listUsersByPermission(Set<String> permissions)..."));
		List<SysUser> Result = new ArrayList<SysUser>();

//		// 找出拥有这些权限的岗位
//		List<SysRolePermission> roleIds = sysRolePermissionMapper.selectList(
//				Wrappers.<SysRolePermission>lambdaQuery()
//						.in(SysRolePermission::getPermissionId, permissions)
//				).stream().collect(Collectors.toList());

		// 找出拥有这些权限的用户
		List<SysUser> someUsers = userMapper.listAllUsersByPermission(permissions);

		// 确认这些用户是否在对应的机构有岗位权限
		for (SysUser theUser : someUsers) {
//			String theUserId = theUser.getUserId();
//System.out.println(String.format("合计 %d 个用户, 确认用户 %s 是否有权限 ===>", someUsers.size(), theUser.getUsername()));

			if (includeSubOrg) {
				// 包含子机构
				if (theUser.getOrgId().equals(orgId) || sysOrgService.getDescendantList(theUser.getOrgId()).contains(orgId)) {
					Result.add(theUser);
				}
			}
			else {
				// 仅用户所在机构
				if (theUser.getOrgId().equals(orgId)) {
					Result.add(theUser);
				}
			}

		}

		return Result;
	}

}
