package digital.laboratory.platform.sys.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import digital.laboratory.platform.sys.dto.UserDTO;
import digital.laboratory.platform.sys.dto.UserInfo;
import digital.laboratory.platform.sys.entity.SysUser;
//import digital.laboratory.platform.admin.api.vo.UserExcelVO;
import digital.laboratory.platform.sys.vo.Staff;
import digital.laboratory.platform.sys.vo.UserVO;

import java.util.List;
import java.util.Set;

/**
 * @author lengleng
 * @date 2019/2/1
 */
public interface SysUserService extends IService<SysUser> {

//	SysUser getUserByUsername(String username); // xxx

	/**
	 * 查询用户信息
	 * @param sysUser 用户
	 * @return userInfo
	 */
	UserInfo getUserInfo(SysUser sysUser);

	/**
	 * 分页查询用户信息(含有岗位信息)
	 * @param page 分页对象
	 * @param userDTO 参数列表
	 * @return
	 */
	IPage getUserWithRolePage(Page page, UserDTO userDTO);

	/**
	 * 删除用户
	 * @param sysUser 用户
	 * @return boolean
	 */
	Boolean removeUserById(SysUser sysUser);

	/**
	 * 更新当前用户基本信息
	 * @param userDto 用户信息
	 * @return Boolean 操作成功返回true,操作失败返回false
	 */
	Boolean updateUserInfo(UserDTO userDto);

	/**
	 * 审核用户
	 * @param userId
	 * @param opcode
	 * @return Boolean 操作成功返回true,操作失败返回false
	 */
	Boolean reviewUser(String userId,boolean opcode) throws Exception;
	/**
	 * 更新指定用户信息
	 * @param userDto 用户信息
	 * @return
	 */
	Boolean updateUser(UserDTO userDto);

	/**
	 * 通过ID查询用户信息
	 * @param id 用户ID
	 * @return 用户信息
	 */
	UserVO getUserVoById(String id);

	/**
	 * 查询上级机构的用户信息
	 * @param username 用户名
	 * @return R
	 */
	List<SysUser> listAncestorUsersByUsername(String username);

	/**
	 * 保存用户信息
	 * @param userDto DTO 对象
	 * @return success/fail
	 */
	Boolean addUser(UserDTO userDto);

	/**
	 * 注册新用户
	 * @param userDto
	 * @return
	 */
	Boolean registerUser(UserDTO userDto);

	/**
	 * 查询全部的用户
	 * @param userDTO 查询条件
	 * @return list
	 */
//	List<UserExcelVO> listUser(UserDTO userDTO);

	/**
	 * excel 导入用户
	 * @param excelVOList excel 列表数据
	 * @param bindingResult 错误数据
	 * @return ok fail
	 */
//	R importUser(List<UserExcelVO> excelVOList, BindingResult bindingResult);

	/**
	 * 根据机构 id 列表查询对应的用户 id 集合
	 * @param orgIds 机构 id 列表
	 * @return userIdList
	 */
	List<String> listUserIdByOrgIds(Set<String> orgIds);

	/**
	 * 查询拥有某权限的用户列表
	 * @param permissions 机构id 集合
	 * @return 用户列表
	 */
	List<SysUser> listUsersByPermission(String orgId, boolean includeSubOrg, List<String> permissions);

	List<Staff> getStaffList(QueryWrapper<Staff> qw);
	//更新前端用户的基本信息数据
	SysUser updateFrontUserInfo(UserDTO userDTOList);
	//添加特殊用户，比如各个市的市一级污水联系人
	List<SysUser> addSpecialUserInfo(List<UserDTO> userDto);
}
