
package digital.laboratory.platform.sys.service.impl;

import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeNode;
import cn.hutool.core.lang.tree.TreeUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.core.constant.CommonConstants;
import digital.laboratory.platform.sys.entity.Dictionary;
import digital.laboratory.platform.sys.vo.DictVueStoreVo;
import digital.laboratory.platform.sys.vo.DictionaryVO;
import digital.laboratory.platform.sys.mapper.DictionaryMapper;
import digital.laboratory.platform.common.core.constant.CacheConstants;
import digital.laboratory.platform.sys.service.DictionaryService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 字典表
 *
 * @author zhangxl
 * @date 2021-12-28
 */
@Service
@RequiredArgsConstructor
public class DictionaryServiceImpl extends ServiceImpl<DictionaryMapper, Dictionary> implements DictionaryService {
	private final DictionaryMapper dictionaryMapper;
//	private final SysDictItemMapper dictItemMapper;

//	/**
//	 * 通过字典类型列出字典项
//	 * @param type
//	 * @return
//	 */
//	@Override
//	public List<Dictionary> listByType(String type) {
//		return dictionaryMapper.listByType(type);
//	}


	@Override
	public List<Tree<String>> treeDictionary(String parentId) {
		List<TreeNode<String>> collect = baseMapper
				.selectList(Wrappers.<Dictionary>lambdaQuery()
						.orderByAsc(Dictionary::getSort))
						.stream()
				.map(getNodeFunction()).collect(Collectors.toList());

		if (StrUtil.isNotBlank(parentId)) {
			return TreeUtil.build(collect, parentId);
		}
		return TreeUtil.build(collect, CommonConstants.PERMISSION_TREE_ROOT_ID);
	}

	@Override
	public List<Tree<String>> getTreeParentDictionary() {

		List<TreeNode<String>> collect=this.baseMapper.listTypes("").stream().map(getNodeFunction()).collect(Collectors.toList());

		return TreeUtil.build(collect, CommonConstants.PERMISSION_TREE_ROOT_ID);
	}

	/**
	 * 返回全部类型, 去重
	 * @return
	 */
	@Override
	public List<Dictionary> listTypes(String module) {
		return dictionaryMapper.listTypes(module);
//		QueryWrapper wrapper = new QueryWrapper();
//		wrapper.select("distinct type").orderByAsc("type");
//		Stream<String> s = dictionaryMapper.selectList(wrapper).stream().map(dict -> ((Dictionary)dict).getType());
//		List<String> list = s.collect(Collectors.toList());
//
//		return list;
	}

	/**
	 * 添加字典
	 * @param dictionary
	 * @return
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Boolean addDictionary(Dictionary dictionary) {
		Dictionary newDictionary = new Dictionary();
		BeanUtils.copyProperties(dictionary, newDictionary);

		this.save(newDictionary);
		//sysOrgRelationService.saveOrgRelation(sysOrg);
		return Boolean.TRUE;
	}

	/**
	 * 根据ID 删除字典
	 * @param id 字典ID
	 * @return
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	@CacheEvict(value = CacheConstants.DIRECTORY_DETAILS, allEntries = true)
	public void removeDictionary(String id) {
		Dictionary dict = this.getById(id);
		// 系统内置
//		Assert.state(!DictTypeEnum.SYSTEM.getType().equals(dict.getSystem()), "系统内置字典项目不能删除");
		dictionaryMapper.deleteById(id);
//		dictItemMapper.delete(Wrappers.<SysDictItem>lambdaQuery().eq(SysDictItem::getDictId, id));
	}

	/**
	 * 更新字典
	 * @param dictionary 字典
	 * @return
	 */
	@Override
	public void updateDictionary(Dictionary dictionary) {
//		Dictionary sysDict = this.getById(dictionary.getId());
		// 系统内置
//		Assert.state(!DictTypeEnum.SYSTEM.getType().equals(sysDict.getSystem()), "系统内置字典项目不能修改");
		this.updateById(dictionary);
	}



	@NotNull
	private Function<Dictionary, TreeNode<String>> getNodeFunction() {
		return dictionary -> {
			TreeNode<String> node = new TreeNode<>();
			node.setId(dictionary.getId());
			node.setParentId(dictionary.getType());
			node.setName(dictionary.getLabel());
			node.setWeight(dictionary.getSort());

			// 扩展属性
			Map<String, Object> extra = new HashMap<>();
			extra.put("module", dictionary.getModule());
			extra.put("type", dictionary.getType());
			extra.put("sort", dictionary.getSort());
			extra.put("code", dictionary.getCode());
			extra.put("label", dictionary.getLabel());
			extra.put("cssClass", dictionary.getCssClass());
			extra.put("listClass", dictionary.getListClass());
			extra.put("defaultValue", dictionary.getDefaultValue());
			extra.put("description", dictionary.getDescription());
			extra.put("comments", dictionary.getComments());
			extra.put("createTime", dictionary.getCreateTime());
			extra.put("updateTime", dictionary.getUpdateTime());
			extra.put("createBy", dictionary.getCreateBy());
			extra.put("updateBy", dictionary.getUpdateBy());

			node.setExtra(extra);
			return node;
		};
	}

	@Override
	public List<DictionaryVO> listDictionaryVO(QueryWrapper<Dictionary> qw) {
		return baseMapper.listDictionaryVO(qw);
	}

	/**
	 * 删除字典，如果该节点下有子节点，那么我们会把子节点也删除掉
	 * @param dictId
	 * @return
	 */
	@Override
	public Boolean deleteDictionary(String dictId) {
		try {
			List<Dictionary> dictionaryChildList =this.list(Wrappers.<Dictionary>query().eq("type",dictId));
			if(dictionaryChildList.size()>0)
			{
				this.removeBatchByIds(dictionaryChildList);
			}
			this.removeDictionary(dictId);
			return true;

		}catch (Exception e)
		{
			return false;
		}
	}

	/**
	 * 添加第三方系统字典数据
	 * @param label
	 * @param id
	 * @return
	 */
	@Override
	public Dictionary addThirdPartyDictionary(String label, String id) {
		List<DictionaryVO> dictionaryVOList = baseMapper.listDictionaryVO(Wrappers.<Dictionary>query()
				.eq(StrUtil.isNotBlank(id), "type", id));
		Integer num = null; // 记录当前分类下有多少个字典

		if (dictionaryVOList == null || dictionaryVOList.size() == 0 || dictionaryVOList.get(0) == null) {
			num = 1;
		} else {
			num = dictionaryVOList.size() + 1;
		}
		Dictionary dictionary = new Dictionary();
		// 设置字典得id
		dictionary.setId(IdWorker.get32UUID());
		dictionary.setCode(String.valueOf(num));
		dictionary.setLabel(label);
		dictionary.setType(id);
		dictionary.setSort(10);
		boolean r = this.save(dictionary);
		if (r) {
			return dictionary;
		}
		else {
			return null;
		}
	}

	@Override
	public List<DictVueStoreVo> getVueStoreLoadDictList(List<String> typeList) {
		long beginTime1=System.currentTimeMillis();
		List<Dictionary>  dictionaryList=this.list(Wrappers.<Dictionary>query()
				.in(typeList.size()>0,"type",typeList));
		long beginTime2=System.currentTimeMillis();

		//将这些查出来的字典按传入的参数分组
		List<DictVueStoreVo> retDictList=new ArrayList<>();
		typeList.forEach(pms_typeID->
		{
			Dictionary dictionaryObj=this.baseMapper.selectOne(Wrappers.<Dictionary>query()
					.eq("id",pms_typeID));
			DictVueStoreVo dictVueStoreVo=new DictVueStoreVo();
			dictVueStoreVo.setDictType(pms_typeID);
			dictVueStoreVo.setDictionary(dictionaryObj);
			List<Dictionary> childDictList=new ArrayList<>();
			dictionaryList.forEach(dictObj->{
				if(dictObj.getType().equals(pms_typeID))
				{
					childDictList.add(dictObj);
				}
			});
			dictVueStoreVo.setChildList(childDictList);
			retDictList.add(dictVueStoreVo);
		});
		long beginTime3=System.currentTimeMillis();
		long st1=beginTime2-beginTime1;
		long st2=beginTime3-beginTime2;
		System.out.println(String.format("beginTime1:{%d} beginTime2:{%d}, beginTime3:{%d},st1:{%d},st2:{%d}",beginTime1,beginTime2,beginTime3,st1,st2));

		return retDictList;
	}
}
