package digital.laboratory.platform.sys.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeNode;
import cn.hutool.core.lang.tree.TreeUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.sys.entity.DrugType;
import digital.laboratory.platform.sys.mapper.DrugTypeMapper;
import digital.laboratory.platform.sys.service.DrugTypeService;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 毒品分类服务实现类
 *
 * @author Zhang Xiaolong created at  2022-06-23
 * @describe 毒品分类服务实现类
 */
@Service
public class DrugTypeServiceImpl extends ServiceImpl<DrugTypeMapper, DrugType> implements DrugTypeService {
    @Override
    public DrugType addDrugType(DrugType drugType) {
           List<DrugType> list=this.list(Wrappers.<DrugType>lambdaQuery()
                .eq(DrugType::getName,drugType.getName()));
           if(list.size()>0){
               throw new RuntimeException("类别名称重复，保存失败");
           }else {
               this.save(drugType);
           }
        return drugType;
    }
    @Override
    public List<Tree<String>> getDrugTypeTreeData(String pid) {
        List<TreeNode<String>> drugTypeList=this.list(Wrappers.<DrugType>lambdaQuery()
                //.eq(DrugType::getPid,pid)
                .orderByAsc(DrugType::getOrderIndex))
                .stream().map(getNodeFunction()).collect(Collectors.toList());
            return TreeUtil.build(drugTypeList, pid);
    }

    @Override
    public String getRootID(String drugTypeId) {
        Assert.notEmpty(drugTypeId,"参数 drugTypeId 不能我空");
        List<DrugType> drugTypeList=this.list();
        String rootId=getPid(drugTypeList,drugTypeId);//
        return rootId;
    }
    @Override
    public List<String> getAllNodeByRootId(String rootId) {
        return  getOffspringId(rootId);
    }

    /**
     * 构建毒品类型的父子关系映射
     */
    @Override
    public Map<String, List<String>> buildDrugTypeHierarchy(List<DrugType> allDrugTypes) {
        Map<String, List<String>> taskChildrenMap = new HashMap<>();
        for (DrugType drugType : allDrugTypes) {
            // 如果任务有父任务ID，则加入到父任务的子任务列表中
            taskChildrenMap.computeIfAbsent(drugType.getPid(), k -> new ArrayList<>()).add(drugType.getId());

        }
        return taskChildrenMap;
    }

    //根据一个子ID查出这个树的根ID
//    private  String getPid(List<DrugType> drugTypeList,String childId){
//        StringBuilder sbRet=new StringBuilder();
//        drugTypeList.forEach(item->{
//                if(item.getId().equals(childId)){
//                    if(item.getPid().equals("0")){
//                        //找到根节点了
//                         sbRet.append(item.getId());
//                    }else{
//                        getPid(drugTypeList,item.getId());
//                    }
//                }
//        });
//        return sbRet.toString();
//    }

    // 根据一个子ID查出这个树的根ID
    private String getPid(List<DrugType> drugTypeList, String childId) {
        for (DrugType item : drugTypeList) {
            if (item.getId().equals(childId)) {
                if ("0".equals(item.getPid())) {
                    // 找到根节点，直接返回根ID
                    return item.getId();
                } else {
                    // 递归查找根ID，注意使用正确的递归参数item.getPid()
                    return getPid(drugTypeList, item.getPid());
                }
            }
        }
        // 如果没有找到对应的子ID或根ID，返回空字符串或抛出异常
        return ""; // 或者抛出异常，视业务逻辑而定
    }

    //根据根ID，分别查询出后代的所有ID
    private List<String> getOffspringId(String rootId){
        List<TreeNode<String>> collect = this
                .list(Wrappers.<DrugType>query()
                        .orderByAsc("order_index"))
                .stream()
                .map(getChildNodeIDFun()).collect(Collectors.toList());
        List<Tree<String>> retData=new ArrayList<>();
        retData= TreeUtil.build(collect, rootId);
        List<String> drugTypeIdList=new ArrayList<>();
        //drugTypeIdList.add(rootId);
        if(retData!=null){
            retData.forEach(item->{
                getChildIdList(item,drugTypeIdList);
            });
        }else{
            drugTypeIdList.add(rootId);
        }
        return drugTypeIdList;
    }
    //遍历树 死循环
//    private void getChildIdList(Tree<String> treeNode,List<String> idList){
//
//        if(treeNode.getChildren()!=null){
//            getChildIdList(treeNode,idList);
//        }else {
//            idList.add(treeNode.getId());
//        }
//    }

    // 遍历树并收集节点ID
    private void getChildIdList(Tree<String> treeNode, List<String> idList) {
        // 首先添加当前节点的ID
        idList.add(treeNode.getId());

        // 如果存在子节点，则对每个子节点递归调用此方法
        if (treeNode.getChildren() != null) {
            for (Tree<String> childNode : treeNode.getChildren()) {
                getChildIdList(childNode, idList);
            }
        }
    }

    private Function<DrugType, TreeNode<String>> getChildNodeIDFun() {
        return drugType -> {
            TreeNode<String> node = new TreeNode<>();
            node.setId(drugType.getId());
            node.setParentId(drugType.getPid());
            return node;
        };
    }
    private Function<DrugType, TreeNode<String>> getNodeFunction() {
        return  drugType -> {
            TreeNode<String> node = new TreeNode<>();
            node.setId(drugType.getId());
            node.setParentId(drugType.getPid());
            node.setName(drugType.getName());
            node.setWeight(drugType.getOrderIndex()+"");
            //设置自定义属性
            Map<String, Object> extra = new HashMap<>();
            extra.put("label",drugType.getName());
            extra.put("value",drugType.getId());
            extra.put("dataObj",drugType);
            node.setExtra(extra);
            return node;
        };
    }
}
