package digital.laboratory.platform.sys.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeNode;
import cn.hutool.core.lang.tree.TreeUtil;
import cn.hutool.core.util.StrUtil;
import digital.laboratory.platform.sys.mapper.SysMenuMapper;
import digital.laboratory.platform.sys.service.SysMenuService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.sys.entity.SysMenu;
import digital.laboratory.platform.common.core.constant.CacheConstants;
import digital.laboratory.platform.common.core.constant.CommonConstants;
import digital.laboratory.platform.common.core.constant.enums.MenuTypeEnum;
import lombok.RequiredArgsConstructor;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.validation.constraints.NotNull;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 菜单权限表 服务实现类
 * </p>
 *
 * @author lengleng
 * @since 2017-10-29
 */
@Service
@RequiredArgsConstructor
public class SysMenuServiceImpl extends ServiceImpl<SysMenuMapper, SysMenu> implements SysMenuService {

//	private final SysRoleMenuMapper sysRoleMenuMapper;

	@Override
//	@Cacheable(value = CacheConstants.MENU_DETAILS, key = "#roleIds  + '_menu'", unless = "#result == null")
	public Set<SysMenu> findMenuByRoleIds(List<String> roleIds) {
//		System.out.println(String.format("findMenuByRoleIds(): roleIds=%s",roleIds.toString()));

//		Set<SysMenu> sms =baseMapper.listMenusByRoleId(roleIds);
//		System.out.println(String.format("findMenuByRoleId(): sms.size()=%d",sms.size()));
//		for (SysMenu sm : sms) {
//			System.out.println(String.format("findMenuByRoleId(): menu menuId=%s name=%s path=%s ",sm.getMenuId(), sm.getName(), sm.getPath()));
//		}
//		return sms;
		return baseMapper.listMenusByRoleId(roleIds);
	}

	@Override
//	@Cacheable(value = CacheConstants.MENU_DETAILS, key = "#roleIds  + '_menu'", unless = "#result == null")
	public Set<SysMenu> findMenuByUserId(String userId) {
//		System.out.println(String.format("findMenuByUserId(): userId=%s",userId));
		return baseMapper.listMenusByUserId(userId);
	}

	/**
	 * 级联删除菜单
	 * @param id 菜单ID
	 * @return true成功, false失败
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	@CacheEvict(value = CacheConstants.MENU_DETAILS, allEntries = true)
	public Boolean removeMenuById(String id) {
		// 查询父节点为当前节点的节点
		List<SysMenu> menuList = this.list(Wrappers.<SysMenu>query().lambda().eq(SysMenu::getParentId, id));

		Assert.isTrue(CollUtil.isEmpty(menuList), "菜单含有下级不能删除");

//		sysRoleMenuMapper.delete(Wrappers.<SysRoleMenu>query().lambda().eq(SysRoleMenu::getMenuId, id));
		// 删除当前菜单
		return this.removeById(id);
	}

	@Override
	@CacheEvict(value = CacheConstants.MENU_DETAILS, allEntries = true)
	public Boolean updateMenuById(SysMenu sysMenu) {
		return this.updateById(sysMenu);
	}

	/**
	 * 根据菜单标识构建树
	 * @param menu 菜单标识
	 * @return 菜单树
	 */
	@Override
	public List<Tree<String>> treeMenuByMenu(String menu) {
		String parent = null;

		// 查找 menu 对应的节点
		SysMenu m = baseMapper.selectOne(Wrappers.<SysMenu>query().lambda().eq(SysMenu::getMenu, menu));
		if (m != null) {
			parent = m.getMenuId();
		}

		// 取 menu 的节点 id, 如果没有找到或者为 null, 则使用 MENU_TREE_ROOT_ID
		if ((parent == null) || (parent.isEmpty())) {
			parent = CommonConstants.MENU_TREE_ROOT_ID;
		}

		List<TreeNode<String>> collect = baseMapper
				.selectList(
						Wrappers.<SysMenu>lambdaQuery().eq(SysMenu::getParentId, parent).orderByAsc(SysMenu::getSort))
				.stream().map(getNodeFunction()).collect(Collectors.toList());

		return TreeUtil.build(collect, parent);
	}


	/**
	 * 构建树查询 1. 不是懒加载情况, 查询全部 2. 是懒加载, 根据parentId 查询 2.1 父节点为空, 则查询ID -1
	 * @param lazy 是否是懒加载
	 * @param parentId 父节点ID
	 * @return
	 */
	@Override
	public List<Tree<String>> treeMenuByParentId(boolean lazy, String parentId) {
		if (!lazy) {
			List<TreeNode<String>> collect = baseMapper
					.selectList(Wrappers.<SysMenu>lambdaQuery().orderByAsc(SysMenu::getSort)).stream()
					.map(getNodeFunction()).collect(Collectors.toList());

			return TreeUtil.build(collect, CommonConstants.MENU_TREE_ROOT_ID);
		}

		String parent = (parentId == null) ? CommonConstants.MENU_TREE_ROOT_ID : parentId;

		List<TreeNode<String>> collect = baseMapper
				.selectList(
						Wrappers.<SysMenu>lambdaQuery().eq(SysMenu::getParentId, parent).orderByAsc(SysMenu::getSort))
				.stream().map(getNodeFunction()).collect(Collectors.toList());

		return TreeUtil.build(collect, parent);
	}

	/**
	 * 查询菜单
	 * @param all 全部菜单
	 * @param parentId 父节点ID
	 * @return
	 */
	@Override
	public List<Tree<String>> filterMenu(Set<SysMenu> all, String parentId) {
		List<TreeNode<String>> collect = all.stream()
				.filter(menu -> MenuTypeEnum.LEFT_MENU.getType().equals(menu.getType()))
				.filter(menu -> StrUtil.isNotBlank(menu.getPath())).map(getNodeFunction()).collect(Collectors.toList());
		String parent = (parentId == null) ? CommonConstants.MENU_TREE_ROOT_ID : parentId;
		return TreeUtil.build(collect, parent);
	}

	@NotNull
	private Function<SysMenu, TreeNode<String>> getNodeFunction() {
		return menu -> {
			TreeNode<String> node = new TreeNode<>();
			node.setId(menu.getMenuId());
			node.setName(menu.getName());
			node.setParentId(menu.getParentId());
			node.setWeight(menu.getSort());

			// 扩展属性
			Map<String, Object> extra = new HashMap<>();
			extra.put("menu", menu.getMenu());
			extra.put("icon", menu.getIcon());
			extra.put("path", menu.getPath());
			extra.put("type", menu.getType());
			extra.put("permissionId", menu.getPermissionId());
			extra.put("label", menu.getName());
			extra.put("sort", menu.getSort());
			extra.put("keepAlive", menu.getKeepAlive());
			node.setExtra(extra);
			return node;
		};
	}

}
