package digital.laboratory.platform.sys.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeNode;
import cn.hutool.core.lang.tree.TreeUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import digital.laboratory.platform.common.core.exception.ValidateCodeException;
import digital.laboratory.platform.sys.entity.Area;
import digital.laboratory.platform.sys.entity.SysOrg;
import digital.laboratory.platform.sys.mapper.SysOrgMapper;
import digital.laboratory.platform.sys.mapper.SysRoleMapper;
import digital.laboratory.platform.sys.service.AreaService;
import digital.laboratory.platform.sys.service.SysOrgService;
import digital.laboratory.platform.common.core.constant.CommonConstants;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.security.util.SecurityUtils;
import digital.laboratory.platform.sys.vo.OrgVO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.validation.constraints.NotNull;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * 机构管理 服务实现类
 * </p>
 *
 * @author lengleng
 * @since 2019/2/1
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class SysOrgServiceImpl extends ServiceImpl<SysOrgMapper, SysOrg> implements SysOrgService {

	private final SysRoleMapper sysRoleMapper;

	private final AreaService areaService;

	/**
	 * 添加信息机构
	 * @param org 机构
	 * @return
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Boolean addOrg(SysOrg org) {
		if (super.count(Wrappers.<SysOrg>lambdaQuery().eq(SysOrg::getAreaCode, org.getAreaCode())) > 0) {
			throw new ValidateCodeException(String.format("该行政区域编码 %s 已经存在！", org.getAreaCode()));
		}
		SysOrg sysOrg = new SysOrg();
		BeanUtils.copyProperties(org, sysOrg);

		System.out.println(String.format("SysOrgServiceImpl.saveOrg() -> org.orgId: %s,  Name=%s, ParentId=%s ", org.getOrgId(), org.getName(), org.getParentId()));
		sysOrg.setOrgId(org.getAreaCode());
		this.save(sysOrg);
		//sysOrgRelationService.saveOrgRelation(sysOrg);
		return Boolean.TRUE;
	}

	/**
	 * 删除机构
	 * @param id 机构 ID
	 * @return 成功、失败
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Boolean removeOrgById(String id) {

		// 查询父节点为当前节点的全部节点
		List<SysOrg> orgList = this.list(Wrappers.<SysOrg>query().lambda().eq(SysOrg::getParentId, id));

		Assert.isTrue(CollUtil.isEmpty(orgList), "机构含有下级不能删除");

		//sysRoleOrgMapper.delete(Wrappers.<SysRoleMenu>query().lambda().eq(SysRoleMenu::getMenuId, id));
		// 删除当前机构
		return this.removeById(id);
/*
		// 级联删除机构
//		List<String> idList = sysOrgRelationService
//				.list(Wrappers.<xSysOrgRelation>query().lambda().eq(xSysOrgRelation::getAncestor, id)).stream()
//				.map(xSysOrgRelation::getDescendant).collect(Collectors.toList());
//
//		if (CollUtil.isNotEmpty(idList)) {
//			this.removeByIds(idList);
//		}
//
//		// 删除机构级联关系
//		sysOrgRelationService.removeOrgRelationById(id);
//		return Boolean.TRUE;
*/

	}

	/**
	 * 更新机构
	 * @param sysOrg 机构信息
	 * @return 成功、失败
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Boolean updateOrgById(SysOrg sysOrg) {
		return this.updateById(sysOrg);
	}


	/**
	 * 返回子机构列表, 不含隔代的后裔
	 * @param orgId 机构id
	 * @return List<String>
	 */
	@Override
	public List<SysOrg> getSubOrg(String orgId) {
//		List<SysOrg>  subOrgs = baseMapper
//				.selectList(Wrappers.<SysOrg>lambdaQuery().orderByAsc(SysOrg::getSort)).stream()
//				.filter(x -> x.getParentId().equals(orgId))
//				.collect(Collectors.toList());

		SysOrg condition = new SysOrg();
		condition.setParentId(orgId);
		List<SysOrg>  subOrgs = baseMapper.selectList(new QueryWrapper<>(condition).orderByAsc("sort"));
		return subOrgs;
	}

	/**
	 * 返回全部后裔的 Id 列表
	 * @param orgId 机构id
	 * @return List<String>
	 */
	@Override
	public List<SysOrg> getDescendantList(String orgId) {
		List<SysOrg> fullOrgs = baseMapper
				.selectList(Wrappers.<SysOrg>lambdaQuery().orderByAsc(SysOrg::getSort)).stream()
				.collect(Collectors.toList());

		return getOrgTree(orgId, fullOrgs);


	}

	/**
	 * 查找指定机构的全部后裔机构列表
	 * @param orgId 机构id
	 * @return List<SysOrg>
	 */
	@Override
	public List<Tree<String>> listDescendantTree(String orgId) {
		List<TreeNode<String>> collect = baseMapper
				.selectList(Wrappers.<SysOrg>lambdaQuery().orderByAsc(SysOrg::getSort)).stream()
				.map(getNodeFunction()).collect(Collectors.toList());
		return TreeUtil.build(collect, orgId);
	}

	/**
	 * 查询全部机构树
	 * @return 树
	 */
	@Override
	public List<Tree<String>> listAllOrgTree() {
		List<TreeNode<String>> collect = baseMapper
				.selectList(Wrappers.<SysOrg>lambdaQuery().orderByAsc(SysOrg::getSort)).stream()
				.map(getNodeFunction()).collect(Collectors.toList());
		//return TreeUtil.build(collect, CommonConstants.ORG_TREE_ROOT_ID);
		return TreeUtil.build(collect, null);
	}

	/**
	 * 查询用户机构树, 单根树, 树为用户所属机构
	 * @return
	 */
	@Override
	public Tree<String> listCurrentUserOrgTree() {
		String orgId = SecurityUtils.getUser().getOrgId();

		if (orgId == null) {
			orgId = CommonConstants.ORG_TREE_ROOT_ID;
		}

		// 这里加载机构的全量数据, 再根据当前用户的 orgId 来构造下属机构树
		// 还有一个办法是一级一级的查下去, 这样数据会少一些, 但是需要查询多次, 速度应该会比较慢
		List<TreeNode<String>> collect = baseMapper
				.selectList(Wrappers.<SysOrg>lambdaQuery().orderByAsc(SysOrg::getSort)).stream()
				.map(getNodeFunction()).collect(Collectors.toList());

//		System.out.println(String.format("---- start of collect -------------"));
//		Iterator<TreeNode<String>> it = collect.listIterator();
//		while(it.hasNext()) {
//			TreeNode<String> node = it.next();
//			System.out.println(String.format("id=%s name=%s", node.getId(), node.getName()));
//		}
//		System.out.println(String.format("----- end of collect ------------"));

		return TreeUtil.buildSingle(collect, orgId);
	}



	/**
	 * 查询用户机构树, 多根树, 去除用户所属机构后的各个分支
	 * @return
	 */
	@Override
	public List<Tree<String>> listCurrentUserOrgTrees() {
		String orgId = SecurityUtils.getUser().getOrgId();

//		String parent = (orgId == null) ? CommonConstants.ORG_TREE_ROOT_ID : orgId;
		SysOrg org = this.getById(orgId);
		String parent = org.getParentId();

//
//		List<TreeNode<String>> collect = baseMapper
//				.selectList(
//						Wrappers.<SysOrg>lambdaQuery().eq(SysOrg::getParentId, parent).orderByAsc(SysOrg::getSort))
//				.stream().map(getNodeFunction()).collect(Collectors.toList());

		// 这里加载机构的全量数据, 再根据当前用户的 orgId 来构造下属机构树
		// 还有一个办法是一级一级的查下去, 这样数据会少一些, 但是需要查询多次, 速度应该会比较慢
		List<TreeNode<String>> collect = baseMapper
				.selectList(Wrappers.<SysOrg>lambdaQuery().orderByAsc(SysOrg::getSort)).stream()
				.map(getNodeFunction()).collect(Collectors.toList());
//		Tree<String> stree = TreeUtil.buildSingle(collect, parent);
		return TreeUtil.build(collect, parent);
	}




	/**
	 * 列举指定机构的祖先和后裔树
	 * 从指定节点开始, 向下是所属的全部机构, 向上是其父节点, 一直回溯到根
	 * @param orgId
	 * @return
	 */
	@Override
	public List<Tree<String>> getAncestorsAndDescendantsOrgTree(String orgId) {
		String parent = (orgId == null) ? CommonConstants.ORG_TREE_ROOT_ID : orgId;

		// 这里加载机构的全量数据, 再根据当前用户的 orgId 来构造下属机构树
		// 还有一个办法是一级一级的查下去, 这样数据会少一些, 但是需要查询多次, 速度应该会比较慢
		List<TreeNode<String>> collect = baseMapper
				.selectList(Wrappers.<SysOrg>lambdaQuery().orderByAsc(SysOrg::getSort)).stream()
				.map(getNodeFunction()).collect(Collectors.toList());

		// 下级树
		List<Tree<String>> subTree = TreeUtil.build(collect, parent);

		// 构造当前节点上级树
		String currentOrgId = parent;
		for (TreeNode<String> node : collect) {
//			if (currentOrgId.equals(CommonConstants.ORG_TREE_ROOT_ID)) {
//				break;  // 遇到根节点了, 中止
//			}

			boolean found=false;
			for (TreeNode<String> tmpNode : collect) {
				if (tmpNode.getId().equals(currentOrgId)) {
					// 上级树
					List<TreeNode<String>> upList = new ArrayList<TreeNode<String>>();
					upList.add(tmpNode);
					currentOrgId = tmpNode.getParentId();
					List<Tree<String>> upTree = TreeUtil.build(upList, currentOrgId);
					upTree.get(0).setChildren(subTree);
					subTree = upTree;
					found = true;
					break;
				}
			}

			if (! found) {
				// 不应该到这里, 如果到了这里, 说明有一个节点的父节点不存在
				break;
			}
		}

		return subTree;
	}

	// 列举用户可代表的机构树
	// 从用户所属节点开始, 向下是所属的全部机构, 向上是其父节点, 一直回溯到根
	@Override
	public List<Tree<String>> getCurrentUserCanRepresentOrgTree() {
		String orgId = SecurityUtils.getUser().getOrgId();

		String parent = (orgId == null) ? CommonConstants.ORG_TREE_ROOT_ID : orgId;

		// 这里加载机构的全量数据, 再根据当前用户的 orgId 来构造下属机构树
		// 还有一个办法是一级一级的查下去, 这样数据会少一些, 但是需要查询多次, 速度应该会比较慢
		List<TreeNode<String>> collect = baseMapper
				.selectList(Wrappers.<SysOrg>lambdaQuery().orderByAsc(SysOrg::getSort)).stream()
				.map(getNodeFunction()).collect(Collectors.toList());

		// 下级树
		List<Tree<String>> subTree = TreeUtil.build(collect, parent);

		// 构造当前节点上级树
		String currentOrgId = parent;
		for (TreeNode<String> node : collect) {
//			if (currentOrgId.equals(CommonConstants.ORG_TREE_ROOT_ID)) {
//				break;  // 遇到根节点了, 中止
//			}

			boolean found=false;
			for (TreeNode<String> tmpNode : collect) {
				if (tmpNode.getId().equals(currentOrgId)) {
					// 上级树
					List<TreeNode<String>> upList = new ArrayList<TreeNode<String>>();
					upList.add(tmpNode);
					currentOrgId = tmpNode.getParentId();
					List<Tree<String>> upTree = TreeUtil.build(upList, currentOrgId);
					upTree.get(0).setChildren(subTree);
					subTree = upTree;
					found = true;
					break;
				}
			}

			if (! found) {
				// 不应该到这里, 如果到了这里, 说明有一个节点的父节点不存在
				break;
			}
		}

		return subTree;
	}




	@NotNull
	private Function<SysOrg, TreeNode<String>> getNodeFunction() {
		return org -> {
			TreeNode<String> node = new TreeNode<>();
			node.setId(org.getOrgId());
			node.setName(org.getName());
			node.setParentId(StrUtil.isBlank(org.getParentId()) ? null : org.getParentId());
			node.setWeight(org.getSort());
			// 扩展属性
			JSONObject json =  (JSONObject) JSON.toJSON(org);
			node.setExtra(json);

			return node;
		};
	}

	/**
	 * 根据提供的 orgId 从 fullOrgs 中查找所有后代, 返回所有后代的列表
	 * @param orgId
	 * @param fullOrgs
	 * @return
	 */
	//@NotNull
	private List<SysOrg> getOrgTree(String orgId, List<SysOrg> fullOrgs) {
		List<SysOrg> subOrgs = new ArrayList<>();
		for (SysOrg org : fullOrgs) {
			if (StrUtil.equalsIgnoreCase(orgId, org.getParentId())) {
				subOrgs.add(org);
			}
		}

		List<SysOrg> descendantsOrgs = new ArrayList<>();
		descendantsOrgs.addAll(subOrgs);
		for (SysOrg org : subOrgs) {
			descendantsOrgs.addAll(getOrgTree(org.getOrgId(), fullOrgs));
		}
		return descendantsOrgs;
	}


	@Override
	public List<OrgVO> listAreaVO(QueryWrapper<SysOrg> qw) {
		return baseMapper.listOrgVO(qw);
	}

	@Override
	public boolean checkOrgNameExist(String orgName) {
		List<SysOrg> orgVOList= baseMapper.selectList(Wrappers.<SysOrg>query().eq("name",orgName));
		if(orgVOList.size()>0)
		{
			return true;
		}else {
			return false;
		}

	}

	/**
	 * 根据机构id返回机构树
	 * @param orgId
	 * @return
	 */
	@Override
	public Tree<String> listOrgTreeByOrgId(String orgId) {
		List<TreeNode<String>> collect = baseMapper
				.selectList(Wrappers.<SysOrg>lambdaQuery().orderByAsc(SysOrg::getSort)).stream()
				.map(getNodeFunction()).collect(Collectors.toList());

		return TreeUtil.buildSingle(collect, orgId);
	}

	/**
	 * 仅提供内部服务使用接口
	 * 根据机构id获取机构的地址省市
	 */
	@Override
	public List<Area> fetchProvinceCityInfoByOrgId(String orgId) {
		SysOrg org = super.getById(orgId);
		Area area = areaService.getOne(Wrappers.<Area>lambdaQuery().eq(Area::getId, org.getAreaCode()));
		// 创建 ObjectMapper 实例
		ObjectMapper objectMapper = new ObjectMapper();

		// 将字符串解析为 List<String>
        List<String> result = null;
        try {
            result = objectMapper.readValue(area.getAncestors(), new TypeReference<List<String>>() {});
        } catch (JsonProcessingException e) {
            log.error("[fetchProvinceCityInfoByOrgId] [{} JSON序列化失败]", area.getAncestors(), e);
        }
        if (CollUtil.isEmpty(result)) {
			return Collections.emptyList();
		}
		return areaService.list(Wrappers.<Area>lambdaQuery().in(Area::getId, result));
	}
}
