package digital.laboratory.platform.sys.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.sys.entity.SysRolePermission;
import digital.laboratory.platform.sys.mapper.SysRolePermissionMapper;
import digital.laboratory.platform.sys.service.SysRolePermissionService;
import digital.laboratory.platform.common.core.constant.CacheConstants;
import lombok.RequiredArgsConstructor;
import org.springframework.cache.CacheManager;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 岗位菜单表 服务实现类
 * </p>
 *
 * @author lengleng
 * @since 2019/2/1
 */
@Service
@RequiredArgsConstructor
public class SysRolePermissionServiceImpl extends ServiceImpl<SysRolePermissionMapper, SysRolePermission> implements SysRolePermissionService {

	private final CacheManager cacheManager;

	/**
	 * @param role
	 * @param roleId 岗位
	 * @param permissionIds 菜单ID拼成的字符串, 每个id之间根据逗号分隔
	 * @return
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Boolean saveRolePermissions(String roleId, String permissionIds) {
		this.remove(Wrappers.<SysRolePermission>query().lambda().eq(SysRolePermission::getRoleId, roleId));

		if (StrUtil.isBlank(permissionIds)) {
			return Boolean.TRUE;
		}
		List<SysRolePermission> rolePermissionList = Arrays.stream(permissionIds.split(StrUtil.COMMA)).map(permissionId -> {
			SysRolePermission rolePermission = new SysRolePermission();
			rolePermission.setRoleId(roleId);
			rolePermission.setPermissionId(permissionId);
			return rolePermission;
		}).collect(Collectors.toList());

		// 清空userinfo
		Objects.requireNonNull(cacheManager.getCache(CacheConstants.USER_DETAILS)).clear();
		// 清空全部的菜单缓存 fix #I4BM58
		Objects.requireNonNull(cacheManager.getCache(CacheConstants.PERMISSION_DETAILS)).clear();
		return this.saveBatch(rolePermissionList);
	}

}
