package digital.laboratory.platform.sys.service.impl;

import digital.laboratory.platform.sys.dto.SysRoleTypeDTO;
import digital.laboratory.platform.sys.dto.SysRoleTypeUserDTO;
import digital.laboratory.platform.sys.entity.SysRoleType;
import digital.laboratory.platform.sys.entity.SysUserRole;
import digital.laboratory.platform.sys.mapper.SysRoleMapper;
import digital.laboratory.platform.sys.service.SysRoleService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.sys.entity.SysRole;
import digital.laboratory.platform.common.core.constant.CacheConstants;
import digital.laboratory.platform.sys.service.SysRoleTypeService;
import digital.laboratory.platform.sys.service.SysUserRoleService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author lengleng
 * @since 2019/2/1
 */
@Service
@RequiredArgsConstructor
public class SysRoleServiceImpl extends ServiceImpl<SysRoleMapper, SysRole> implements SysRoleService {

//	private final SysRoleMenuMapper sysRoleMenuMapper;

    @Autowired
    private SysRoleTypeService sysRoleTypeService;

    @Autowired
    private SysUserRoleService sysUserRoleService;

    /**
     * 通过权限查询岗位
     *
     * @param permission 权限名
     * @return 岗位的集合
     */
    @Override
    @Cacheable(value = CacheConstants.ROLE_DETAILS, key = "#permission  + '_roles'", unless = "#result == null")
    public Set<SysUserRole> findUserRolesByPermission(String userId, String permission) {
        return baseMapper.listUserRolesByPermission(userId, permission);
    }


    /**
     * 通过岗位ID, 删除岗位,并清空岗位菜单缓存
     *
     * @param id
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    @CacheEvict(value = CacheConstants.PERMISSION_DETAILS, allEntries = true)
    public Boolean removeRoleById(String id) {
//		sysRoleMenuMapper.delete(Wrappers.<SysRoleMenu>update().lambda().eq(SysRoleMenu::getRoleId, id));
        return this.removeById(id);
    }

    /*
     * 为岗位类别分配岗位*/
    @Override
    public boolean addRoleType(SysRoleTypeDTO sysRoleTypeDTO) {

        SysRoleType roleType = sysRoleTypeService.getOne(Wrappers.<SysRoleType>query().eq("id", sysRoleTypeDTO.getRoleTypeId()));

        if (roleType == null) {
            throw new RuntimeException(String.format("选择的岗位类别有误"));
        }

        List<SysRole> sysRoleList = sysRoleTypeDTO.getSysRoleList();

        for (SysRole sysRole : sysRoleList) {
            sysRole.setRoleType(roleType.getId());
        }
        return this.updateBatchById(sysRoleList);
    }

    /*
     * 移除岗位的岗位类别*/
    @Override
    public boolean removeRoleType(List<SysRole> sysRoleList) {

        for (SysRole sysRole : sysRoleList) {
            sysRole.setRoleType(null);
        }
        return this.updateBatchById(sysRoleList);
    }

    @Override
    @Transactional
    public boolean addUserRoleType(SysRoleTypeUserDTO sysRoleTypeUserDTO) {

        List<String> sysUserList = sysRoleTypeUserDTO.getSysUserList();

        SysRoleType sysRoleType = sysRoleTypeService.getById(sysRoleTypeUserDTO.getRoleTypeId());

        if (sysRoleType == null) {
            throw new RuntimeException(String.format("未查询到该岗位类别的信息"));
        }
        List<SysRole> sysRoleList=getRoleByUserIdAndTypes(sysUserList,sysRoleTypeUserDTO.getRoleTypeId());

        if (sysRoleList == null||sysRoleList.size() == 0) {

            throw new RuntimeException(String.format("该岗位类别下未存在其他岗位"));
        }
        List<SysUserRole> sysUserRoles = new ArrayList<>();
        for (SysRole sysRole : sysRoleList) {
            //先将这个岗位之前里包含的用户去掉，然后再把新用户装进去
            sysUserRoleService.remove(Wrappers.<SysUserRole>query()
                    .eq("role_id", sysRole.getRoleId())
                    .in("user_id",sysUserList));

            if (sysUserList != null) {
                for (String sysUser : sysUserList) {

                    SysUserRole userRole = new SysUserRole();
                    userRole.setUserId(sysUser);
                    userRole.setRoleId(sysRole.getRoleId());

                    sysUserRoles.add(userRole);
                }
            }
        }
        if (sysUserList != null && sysUserList.size() != 0) {
            return sysUserRoleService.saveBatch(sysUserRoles);
        } else return true;

    }
    //根据userID和typeID查出刚问
    private List<SysRole> getRoleByUserIdAndTypes(List<String> userIdList,String roleType){
        List<SysUserRole> roleList=sysUserRoleService.list(Wrappers.<SysUserRole>lambdaQuery()
                .in(SysUserRole::getUserId,userIdList));
        List<String> roleIdList1=new ArrayList<>();//根据用户ID获取到的，
        roleList.forEach(item->{
            roleIdList1.add(item.getRoleId());
        });
        List<String> typeIdList=sysRoleTypeService.getTreeChild(roleType);
        List<SysRole> rolesList= this.list(Wrappers.<SysRole>lambdaQuery()
                .in(SysRole::getRoleType,typeIdList)
                .in(SysRole::getRoleId,roleIdList1));//在提供的类型中，在提供的人员中 2个条件成立的，才算是符合这次操作的
        return rolesList;
    }
}

