package digital.laboratory.platform.sys.service.impl;

import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeNode;
import cn.hutool.core.lang.tree.TreeUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.sys.entity.SysRoleType;
import digital.laboratory.platform.sys.mapper.SysRoleTypeMapper;
import digital.laboratory.platform.sys.service.SysRoleService;
import digital.laboratory.platform.sys.service.SysRoleTypeService;
import digital.laboratory.platform.sys.vo.SysRoleTypeVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class SysRoleTypeServiceImpl extends ServiceImpl<SysRoleTypeMapper, SysRoleType> implements SysRoleTypeService {

    @Autowired
    private SysRoleService sysRoleService;

    //新增岗位类别信息
    @Override
    public SysRoleType addRoleType(SysRoleType sysRoleType) {
        //判断是否重复添加岗位类别
        SysRoleType roleType = this.getOne(Wrappers.<SysRoleType>query().eq("type_name", sysRoleType.getTypeName()));

        if (roleType != null) {
            throw new RuntimeException(String.format("该岗位类别已存在，请勿重复添加"));
        }
        //判断是否具有父ID
        if (!StrUtil.isNotBlank(sysRoleType.getParentId())) {

            sysRoleType.setParentId("0");
        } else {
            SysRoleType roleType1 = this.getOne(Wrappers.<SysRoleType>query().eq("id", sysRoleType.getParentId()));
            //判断通过父岗位是否存在
            if (roleType1 == null) {

                throw new RuntimeException(String.format("未查询到当前岗位的父岗位，请重新选择"));
            }
        }
        sysRoleType.setId(IdWorker.get32UUID().toUpperCase());

        if (this.save(sysRoleType)) {
            return sysRoleType;
        } else throw new RuntimeException(String.format("保存失败"));

    }

    @Override
    //修改岗位类别信息
    public SysRoleType editRoleType(SysRoleType sysRoleType) {
        //判断是否重复添加岗位类别
        SysRoleType roleType = this.getOne(Wrappers.<SysRoleType>query().eq("typeName", sysRoleType.getTypeName()));

        if (roleType != null) {
            throw new RuntimeException(String.format("该岗位类别已存在，请勿重复添加"));
        }
        //判断是否具有父ID
        if (!StrUtil.isNotBlank(sysRoleType.getParentId())) {

            sysRoleType.setParentId("0");
        } else {
            SysRoleType roleType1 = this.getOne(Wrappers.<SysRoleType>query().eq("id", sysRoleType.getParentId()));
            //判断通过父岗位是否存在
            if (roleType1 == null) {

                throw new RuntimeException(String.format("未查询到当前岗位的父岗位，请重新选择"));
            }
        }

        if (this.updateById(sysRoleType)) {
            return sysRoleType;
        } else throw new RuntimeException(String.format("修改失败"));

    }

    @Override//删除岗位类别，
    @Transactional
    public boolean delSysRoleType(String id) {

        SysRoleType roleType = this.getById(id);

        List<SysRoleType> sysRoleTypeList = this.list(Wrappers.<SysRoleType>query().eq("parent_id", roleType.getId()));
        /*
         * 如果存在子节点，则会一同删除子节点的类别*/
        if (sysRoleTypeList.size() != 0 & sysRoleTypeList != null) {

            this.removeBatchByIds(sysRoleTypeList);
        }

        return (this.removeById(roleType));
    }

    @Override//通过岗位类别ID查询具体岗位类别
    public SysRoleTypeVO getSysRoleType(String id) {

        return baseMapper.getSysRoleType(id);
    }

    @Override//查询所有的岗位类别
    public List<SysRoleTypeVO> getSysRoleTypeList() {

        return baseMapper.getSysRoleTypeList(Wrappers.query().orderByDesc("create_time"));
    }

    @Override//树查询接口
    public List<Tree<String>> treeSysRoleType(String parentId) {
        List<TreeNode<String>> collect = baseMapper
                .getSysRoleTypeList(Wrappers.<SysRoleType>query().orderByAsc("create_time"))
                .stream()
                .map(getNodeFunction()).collect(Collectors.toList());

        if (StrUtil.isNotBlank(parentId)) {
            return TreeUtil.build(collect, parentId);
        }
        return TreeUtil.build(collect, "0");
    }

    @Override//树查询接口
    public List<Tree<String>> getTreeParentSysRoleType() {

        List<TreeNode<String>> collect = this.list().stream().map(getNodeFunction()).collect(Collectors.toList());

        return TreeUtil.build(collect, "0");
    }

    @NotNull
    private Function<SysRoleType, TreeNode<String>> getNodeFunction() {
        return sysRoleType -> {
            TreeNode<String> node = new TreeNode<>();
            node.setId(sysRoleType.getId());
            node.setName(sysRoleType.getTypeName());
            node.setParentId(sysRoleType.getParentId());
            node.setWeight(sysRoleType.getComments());
            Map<String, Object> extra = new HashMap<>();
            extra.put("label", sysRoleType.getTypeName());
            extra.put("value", sysRoleType.getId());
            extra.put("cascadeValue", sysRoleType.getCascadeValue());
            extra.put("orderIndex", sysRoleType.getOrderIndex());
            extra.put("comments", sysRoleType.getComments());
            node.setExtra(extra);
            return node;
        };
    }
    private Function<SysRoleType, TreeNode<String>> getChildNodeIDFun() {
        return sysRoleType -> {
            TreeNode<String> node = new TreeNode<>();
            node.setId(sysRoleType.getId());
            node.setParentId(sysRoleType.getParentId());
            return node;
        };
    }
    @Override
    public  List<String> getTreeChild(String parentID) {
        List<TreeNode<String>> collect = this
                .list(Wrappers.<SysRoleType>query().orderByAsc("order_index"))
                .stream()
                .map(getChildNodeIDFun()).collect(Collectors.toList());
        List<Tree<String>> retData=new ArrayList<>();
        if(StrUtil.isNotBlank(parentID)) {
            retData= TreeUtil.build(collect, parentID);
        }else {
            retData= TreeUtil.build(collect, "0");
        }
        //如果 retData为空，就说明没子节点，那么我们只需要返回他自己ID即可
        List<String> retIdList=new ArrayList<>();
        //retIdList.add(parentID);
        if(retData!=null){
            retData.forEach(item->{
                getChildIdList(item,retIdList);
            });
        }else{
            retIdList.add(parentID);
        }
        return retIdList;
    }
    public  void getChildIdList(Tree<String> treeNode,List<String> idList)
    {
        if(treeNode.getChildren()!=null){
            for (Tree<String> child : treeNode.getChildren()) {
                getChildIdList(child,idList);
            }
        }else {
            idList.add(treeNode.getId());
            //System.out.println("查询到的ID："+treeNode.getId());
        }

    }
}
