package digital.laboratory.platform.sys.service.impl;

import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import digital.laboratory.platform.common.core.constant.CacheConstants;
import digital.laboratory.platform.common.core.constant.enums.PermissionTypeEnum;
import digital.laboratory.platform.sys.dto.UserDTO;
import digital.laboratory.platform.sys.dto.UserInfo;
import digital.laboratory.platform.sys.entity.*;
import digital.laboratory.platform.sys.mapper.*;
import digital.laboratory.platform.sys.service.*;
import digital.laboratory.platform.sys.vo.Staff;
import digital.laboratory.platform.sys.vo.UserVO;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author Zhang Xiaolong
 * @since 2022-01-20
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements SysUserService {

    //private static final PasswordEncoder ENCODER = new BCryptPasswordEncoder();
    private static final PasswordEncoder ENCODER = new org.springframework.security.crypto.password.MessageDigestPasswordEncoder("MD5");

    private final SysRoleMapper sysRoleMapper;
    private final SysRolePermissionMapper sysRolePermissionMapper;
    private final SysOrgMapper sysOrgMapper;
    private final SysOrgService sysOrgService;
    private final SysMenuService sysMenuService;
    private final SysPermissionService sysPermissionService;
    private final SysUserRoleMapper sysUserRoleMapper;

    @Resource
    private SysUserRoleService sysUserRoleService;
    private final StaffMapper staffMapper;

    /**
     * 添加新用户
     *
     * @param userDto DTO 对象
     * @return success/fail
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean addUser(UserDTO userDto) {
//		InsidePerson person = userDto.getPersonInfo();
//		if ((person.getPersonId() == null) || (person.getPersonId().isEmpty())) {
//			// 人员不存在, 需要添加
//			if (person.getName().isEmpty() ||
//				person.getIdentificationCardNo().isEmpty()) {
//				return Boolean.TRUE;
//			}
//			insidePersonService.save(person);
//			userDto.setPersonId(person.getPersonId());
//		}

        SysUser sysUser = new SysUser();
        BeanUtils.copyProperties(userDto, sysUser);
        sysUser.setPassword(ENCODER.encode(userDto.getPassword()));

        baseMapper.insert(sysUser);

        //插入岗位信息
        System.out.println(String.format("添加新用户, username=%s roleids=%s", userDto.getUsername(), userDto.getRoleIds()));
        for (String roleId : userDto.getRoleIds()) {
            SysUserRole ur = new SysUserRole();
            //ur.setUserId(userDto.getUserId());// 2023-02-10  这里有一个BUG，在userDto中是没有userID的值的，因为这个对象是前台传递来的，
            // 而userID是在保存的时候生成的，
            ur.setUserId(sysUser.getUserId());
            ur.setRoleId(roleId);
            sysUserRoleMapper.insert(ur);
        }
//		if (userDto.getUserRoleList() != null) {
//			userDto.getUserRoleList().stream().map(userRole -> {
//				return userRole;
//			}).forEach(sysUserRoleMapper::insert);
//		}

        return Boolean.TRUE;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean registerUser(UserDTO userDto) {
        SysUser sysUser = new SysUser();
        BeanUtils.copyProperties(userDto, sysUser);
        sysUser.setPassword(ENCODER.encode(userDto.getPassword()));
        sysUser.setLockFlag(false);//设置用户的锁定状态为未锁定
        sysUser.setApproved(false);//设置新注册的用户是未审核状态
        baseMapper.insert(sysUser);
        //赋予用户一个基础权限
        return Boolean.TRUE;
    }

    /**
     * 通过查用户的全部信息
     *
     * @param sysUser 用户
     * @return
     */
    @Override
    public UserInfo getUserInfo(SysUser sysUser) {
//		System.out.println(String.format("public UserInfo getUserInfo(SysUser sysUser)..."));
        UserInfo userInfo = new UserInfo();
        userInfo.setSysUser(sysUser);

        // 加载员工信息, 如果不是员工, 信息为空
        PartialStaff partialStaff = staffMapper.selectById(sysUser.getUserId());
        userInfo.setPartialStaff(partialStaff);

        // 设置岗位列表
        List<SysRole> roleList = sysRoleMapper.listRolesByUserId(sysUser.getUserId());
        userInfo.setRoleList(roleList);

        // 设置岗位列表 (ID)
        List<String> roleIds = roleList.stream().map(SysRole::getRoleId).collect(Collectors.toList());
        userInfo.setRoles(ArrayUtil.toArray(roleIds, String.class));

        // 设置权限列表(menu.permission)
//		Set<String> permissions = sysMenuService.findMenuByRoleId(roleIds).stream()
//				.filter(m -> MenuTypeEnum.BUTTON.getType().equals(m.getType()))
//				.filter(m -> StrUtil.isNotBlank(m.getPermission())).map(SysMenu::getPermission)
//				.collect(Collectors.toSet());
//		userInfo.setPermissions(ArrayUtil.toArray(permissions, String.class));

        // 设置权限列表(menu.permission)
        Set<String> permissions = sysPermissionService.findPermissionByRoleId(roleIds).stream()
                .filter(m -> PermissionTypeEnum.BACK_SIDE.getType().equals(m.getType()))
                .filter(m -> StrUtil.isNotBlank(m.getPermission()))
                .map(SysPermission::getPermission)
                .collect(Collectors.toSet());

        String[] permissionList = ArrayUtil.toArray(permissions, String.class);
        Arrays.sort(permissionList);
        userInfo.setPermissions(permissionList);

        // 设置所属机构信息
        if (StrUtil.isBlank(sysUser.getOrgId())) {
            userInfo.setOrg(null);
        } else {
            SysOrg org = sysOrgService.getById(sysUser.getOrgId());
            userInfo.setOrg(org);
        }

        return userInfo;
    }

    /**
     * 分页查询用户信息(含有岗位信息)
     *
     * @param page    分页对象
     * @param userDTO 参数列表
     * @return
     */
    @Override
    public IPage getUserWithRolePage(Page page, UserDTO userDTO) {
        System.out.println(String.format("public IPage getUserWithRolePage(Page page, UserDTO userDTO)..."));
        //加入支持的字段，目前支持姓名，登陆用户名，审核状态，账户锁定状态
        QueryWrapper<UserVO> qw = Wrappers.<UserVO>query()
                .like(StrUtil.isNotBlank(userDTO.getUsername()), "u.username", userDTO.getUsername())
                .like(StrUtil.isNotBlank(userDTO.getName()), "u.name", userDTO.getName())
                .eq(StrUtil.isNotBlank(userDTO.getPoliceNo()), "u.police_no", userDTO.getPoliceNo())
                .eq(userDTO.getApproved() != null, "u.approved", userDTO.getApproved())
                .eq(userDTO.getLockFlag() != null, "u.lock_flag", userDTO.getLockFlag())
                .orderByAsc("u.username");
        return baseMapper.getUserVosPage(page, qw);
    }

    /**
     * 通过ID查询用户信息
     *
     * @param id 用户ID
     * @return 用户信息
     */
    @Override
    public UserVO getUserVoById(String id) {
        System.out.println(String.format("public UserVO getUserVoById(String id)..."));
        UserVO v = baseMapper.getUserVoById(id);
        return v;
        // return baseMapper.getUserVoById(id);
    }

    /**
     * 删除用户
     *
     * @param sysUser 用户
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    @CacheEvict(value = CacheConstants.USER_DETAILS, key = "#sysUser.username")
    public Boolean removeUserById(SysUser sysUser) {
        sysUserRoleMapper.deleteByUserId(sysUser.getUserId());  // 删除用户岗位
        this.removeById(sysUser.getUserId());                   // 删除用户
        return Boolean.TRUE;
    }

//	@Override
//	@CacheEvict(value = CacheConstants.SYS_USER, key = "#username")
//	public SysUser getUserByUsername(String username) { // xxx
//		return baseMapper.getUserByUsername(username); // xxx
//
//	}

    @Override
    @Transactional(rollbackFor = Exception.class)
    //@CacheEvict(value = CacheConstants.USER_DETAILS, key = "#userDto.username")
    public Boolean updateUserInfo(UserDTO userDto) {
        UpdateWrapper<SysUser> updateWrapper = new UpdateWrapper<SysUser>();
        //如果是修改密码
        if (StrUtil.isNotBlank(userDto.getNewpassword1())) {
            UserVO userVO = baseMapper.getUserVoById(userDto.getUserId());
            Assert.isTrue(ENCODER.matches(userDto.getPassword(), userVO.getPassword()), "原密码错误, 修改失败");
            updateWrapper.set("password", ENCODER.encode(userDto.getNewpassword1()));
        }
        updateWrapper.eq("user_id", userDto.getUserId());
        updateWrapper.set(StrUtil.isNotBlank(userDto.getUsername()), "username", userDto.getUsername());
        updateWrapper.set(StrUtil.isNotBlank(userDto.getName()), "name", userDto.getName());
        updateWrapper.set(userDto.getApproved() != null, "approved", userDto.getApproved());
        updateWrapper.set(StrUtil.isNotBlank(userDto.getOrgId()), "org_id", userDto.getOrgId());
        updateWrapper.set(StrUtil.isNotBlank(userDto.getAddress()), "address", userDto.getAddress());
        updateWrapper.set(userDto.getEnabled() != null, "enabled", userDto.getEnabled());
        updateWrapper.set(userDto.getPoliceType() != null, "police_type", userDto.getPoliceType());
        updateWrapper.set(StrUtil.isNotBlank(userDto.getPoliceNo()), "police_no", userDto.getPoliceNo());
        updateWrapper.set(StrUtil.isNotBlank(userDto.getDepartment()), "department", userDto.getDepartment());
        updateWrapper.set(StrUtil.isNotBlank(userDto.getPosition()), "position", userDto.getPosition());
        updateWrapper.set(StrUtil.isNotBlank(userDto.getAvatar()), "avatar", userDto.getAvatar());
        updateWrapper.set(userDto.getLockFlag() != null, "lock_flag", userDto.getLockFlag());
        updateWrapper.set(userDto.getGender() != null, "gender", userDto.getGender());
        updateWrapper.set(StrUtil.isNotBlank(userDto.getCertificateType()), "certificate_type", userDto.getCertificateType());
        updateWrapper.set(StrUtil.isNotBlank(userDto.getCertificateNo()), "certificate_no", userDto.getCertificateNo());

        return this.update(updateWrapper);
    }

    /**
     * 审核用户
     *
     * @param userId
     * @param opcode
     * @return
     */
    @Override
    public Boolean reviewUser(String userId, boolean opcode) throws Exception {
        SysUser sysUser = baseMapper.selectById(userId);
        if (sysUser == null) {
            throw new Exception("用户不存在");
        }
        sysUser.setApproved(opcode);
        int ret = baseMapper.updateById(sysUser);
        if (ret > 0) {
            return true;
        } else {
            return false;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    //@CacheEvict(value = CacheConstants.USER_DETAILS, key = "#userDto.username")
    public Boolean updateUser(UserDTO userDto) {
        SysUser sysUser = new SysUser();
        BeanUtils.copyProperties(userDto, sysUser);
//		sysUser.setUpdateTime(LocalDateTime.now());
        //这个判断有问题，因为是修改用户，那么password必然不为空的，所以这里相当于又把原来的加密字符串重新加了一次密，下次登录当然就不正确了   2023-02-23 xyl 修改
        //这里正常情况是不会返回密码到客户端的，但是目前接口方面的问题，暂时让客户端不接收密码字段，所以将其打开  20230324-xyl
        if (StrUtil.isNotBlank(userDto.getPassword())) {
            sysUser.setPassword(ENCODER.encode(userDto.getPassword()));
        }
        //如果newpassword1 字段不为空，则可能是在做修改密码操作，需要判断原密码是否正确
        if (StrUtil.isNotBlank(userDto.getNewpassword1())) {
            UserVO userVO = baseMapper.getUserVoByUsername(userDto.getUsername());
            Assert.isTrue(ENCODER.matches(userDto.getPassword(), userVO.getPassword()), "原密码错误, 修改失败");
            sysUser.setPassword(ENCODER.encode(userDto.getNewpassword1()));
        }
        this.updateById(sysUser);
        sysUserRoleMapper.delete(Wrappers.<SysUserRole>update().lambda().eq(SysUserRole::getUserId, userDto.getUserId()));

        for (String roleId : userDto.getRoleIds()) {
            SysUserRole ur = new SysUserRole();
            ur.setUserId(userDto.getUserId());
            ur.setRoleId(roleId);
            sysUserRoleMapper.insert(ur);
        }
//		if (userDto.getUserRoleList() != null) {
//			userDto.getUserRoleList().forEach(userRole -> {
//				userRole.insert();
//			});
//		}

        return Boolean.TRUE;
    }

    /**
     * 查询上级机构的用户信息
     *
     * @param username 用户名
     * @return R
     */
    @Override
    public List<SysUser> listAncestorUsersByUsername(String username) {
        System.out.println(String.format("public List<SysUser> listAncestorUsersByUsername(String username)..."));
        SysUser user = this.getOne(Wrappers.<SysUser>query().lambda().eq(SysUser::getUsername, username));
        //SysUser user = this.getUserByUsername(username);

        SysOrg sysOrg = sysOrgMapper.selectById(user.getOrgId());
        if (sysOrg == null) {
            return null;
        }

        String parentId = sysOrg.getParentId();
        return this.list(Wrappers.<SysUser>query().lambda().eq(SysUser::getOrgId, parentId));
    }

//	/**
//	 * 查询全部的用户
//	 * @param userDTO 查询条件
//	 * @return list
//	 */
//	@Override
//	public List<UserExcelVO> listUser(UserDTO userDTO) {
//		List<UserVO> voList = baseMapper.selectVoList(userDTO);
//		// 转换成execl 对象输出
//		List<UserExcelVO> userExcelVOList = voList.stream().map(userVO -> {
//			UserExcelVO excelVO = new UserExcelVO();
//			BeanUtils.copyProperties(userVO, excelVO);
//			//String roleNameList = userVO.getRoleList().stream().map(SysRole::getRoleName)
//			String roleNameList = userVO.getUserRoleList().stream().map(SysUserRole::toString)
//					.collect(Collectors.joining(StrUtil.COMMA));
//			excelVO.setRoleNameList(roleNameList);
//			return excelVO;
//		}).collect(Collectors.toList());
//		return userExcelVOList;
//	}

//	/**
//	 * excel 导入用户, 插入正确的 错误的提示行号
//	 * @param excelVOList excel 列表数据
//	 * @param bindingResult 错误数据
//	 * @return ok fail
//	 */
//	@Override
//	public R importUser(List<UserExcelVO> excelVOList, BindingResult bindingResult) {
//		// 通用校验获取失败的数据
//		List<ErrorMessage> errorMessageList = (List<ErrorMessage>) bindingResult.getTarget();
//
//		// 个性化校验逻辑
//		List<SysUser> userList = this.list();
//		List<SysOrg> orgList = sysOrgMapper.selectList(Wrappers.emptyWrapper());
//		List<SysRole> roleList = sysRoleMapper.selectList(Wrappers.emptyWrapper());
//
//		// 执行数据插入操作 组装 UserDto
//		for (int i = 0; i < excelVOList.size(); i++) {
//			UserExcelVO excel = excelVOList.get(i);
//			Set<String> errorMsg = new HashSet<>();
//			// 校验用户名是否存在
//			boolean exsitUserName = userList.stream()
//					.anyMatch(sysUser -> excel.getUsername().equals(sysUser.getUsername()));
//
//			if (exsitUserName) {
//				errorMsg.add(String.format("%s 用户名已存在", excel.getUsername()));
//			}
//
//			// 判断输入的机构名称列表是否合法
//			Optional<SysOrg> orgOptional = orgList.stream()
//					.filter(org -> excel.getOrgName().equals(org.getName())).findFirst();
//			if (!orgOptional.isPresent()) {
//				errorMsg.add(String.format("%s 机构名称不存在", excel.getOrgName()));
//			}
//
//			// 判断输入的岗位名称列表是否合法
//			List<String> roleNameList = StrUtil.split(excel.getRoleNameList(), StrUtil.COMMA);
//			List<SysRole> roleCollList = roleList.stream()
//					.filter(role -> roleNameList.stream().anyMatch(name -> role.getRoleName().equals(name)))
//					.collect(Collectors.toList());
//
//			if (roleCollList.size() != roleNameList.size()) {
//				errorMsg.add(String.format("%s 岗位名称不存在", excel.getRoleNameList()));
//			}
//
//			// 数据合法情况
//			if (CollUtil.isEmpty(errorMsg)) {
//				insertExcelUser(excel, orgOptional, roleCollList);
//			}
//			else {
//				// 数据不合法情况
//				errorMessageList.add(new ErrorMessage((long) (i + 2), errorMsg));
//			}
//
//		}
//
//		if (CollUtil.isNotEmpty(errorMessageList)) {
//			return R.failed(errorMessageList);
//		}
//		return R.ok();
//	}

    @Override
    public List<String> listUserIdByOrgIds(Set<String> orgIds) {
        return this.listObjs(
                Wrappers.lambdaQuery(SysUser.class).select(SysUser::getUserId).in(SysUser::getOrgId, orgIds),
                String.class::cast);
    }

    /**
     * 查询对指定的机构拥有某些权限的用户列表。
     * <p>
     * 还可以通过 includeSubOrg, 对用户进行筛选。
     * includeSubOrg 为 true 时, 如果指定机构是某用户所属机构的下属机构时该用户有效;
     * includeSubOrg 为 false 时, 只有当指定机构是某用户所属机构时该用户有效;
     *
     * @param orgId         机构 id
     * @param includeSubOrg 是否包含子机构
     * @param permissions   权限集合
     * @return 用户列表
     */
    @Override
    public List<SysUser> listUsersByPermission(String orgId, boolean includeSubOrg, List<String> permissions) {
        System.out.println(String.format("public List<SysUser> listUsersByPermission(Set<String> permissions)..."));
        List<SysUser> Result = new ArrayList<SysUser>();

//		// 找出拥有这些权限的岗位
//		List<SysRolePermission> roleIds = sysRolePermissionMapper.selectList(
//				Wrappers.<SysRolePermission>lambdaQuery()
//						.in(SysRolePermission::getPermissionId, permissions)
//				).stream().collect(Collectors.toList());

        // 找出拥有这些权限的用户
        List<SysUser> someUsers = baseMapper.listAllUsersByPermission(permissions);

        // 确认这些用户是否在对应的机构有岗位权限
        for (SysUser theUser : someUsers) {
//			String theUserId = theUser.getUserId();
//System.out.println(String.format("合计 %d 个用户, 确认用户 %s 是否有权限 ===>", someUsers.size(), theUser.getUsername()));

            if (includeSubOrg) {
                // 包含子机构
                if (theUser.getOrgId().equals(orgId) || sysOrgService.getDescendantList(theUser.getOrgId()).contains(orgId)) {
                    Result.add(theUser);
                }
            } else {
                // 仅用户所在机构
                if (theUser.getOrgId().equals(orgId)) {
                    Result.add(theUser);
                }
            }

        }

        return Result;
    }


    @Override
    public List<Staff> getStaffList(QueryWrapper<Staff> qw) {
        return baseMapper.getStaffList(qw);
    }
    //-----------------------------add 23-07-06

    @Override
    public SysUser updateFrontUserInfo(UserDTO userDto) {
        //需更新的额属性有：用户姓名，用户电话，用户证件类型，用户证件号码
        SysUser u = this.getById(userDto.getUserId());
        u.setName(userDto.getName());
        u.setMobile(userDto.getMobile());
        u.setCertificateType(userDto.getCertificateType());
        u.setCertificateNo(userDto.getCertificateNo());
        if (this.updateById(u)) {
            return u;
        } else {
            return null;
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<SysUser> addSpecialUserInfo(List<UserDTO> userDTOList) {
        ArrayList<SysUser> sysUsers = new ArrayList<>();
        for (UserDTO userDto : userDTOList) {
            //添加一个可以登录的用户
            SysUser u = new SysUser();
            u.setUserId(IdWorker.get32UUID());
            u.setUsername(userDto.getUsername());
            u.setPassword(ENCODER.encode(userDto.getPassword()));
            u.setGender(userDto.getGender());
            u.setOrgId(userDto.getOrgId());
            u.setEnabled(true);
            u.setApproved(true);
            //需要给这个用户授权
            List<String> roleIds = userDto.getRoleIds();
            List<SysUserRole> uroles = new ArrayList<>();
            roleIds.forEach(item -> {
                SysUserRole ur = new SysUserRole();
                ur.setUserId(u.getUserId());
                ur.setRoleId(item);
                uroles.add(ur);
            });
            sysUserRoleService.saveBatch(uroles);
            sysUsers.add(u);
        }
        if (this.saveBatch(sysUsers)) {
            return sysUsers;
        } else {
            return null;
        }
    }
}
